/*{{{  usage.c*/
#include "impdo.h"
#include "intrin.h"
#include "output.h"	/* for 'earlier_error' */
#include "type.h"
#include "typedef.h"
#include "symbol.h"
#include "unit.h"	/* only for 'this_unitp->name' -- used once! */

/*{{{  global decls*/

static char
	*str_alloc =		"an allocatable array",
	*str_pointer =		"a pointer",
	*str_normal_var =	"an initialised variable",
	*str_param =		"a named constant",
	*str_func =		"a function",
	*str_subr =		"a subroutine",
	*str_module =		"a module",
	*str_program =		"the program name",
	*str_block_data =	"the block data name",
	*str_derived_type =	"a derived type name",
	*str_construct =	"a construct name",
	*str_processors =	"a processor arrangement",
	*str_template =		"a template",
	*str_external =		"an external procedure",
	*str_intrinsic =	"an intrinsic procedure",
	*str_namelist_group =	"a namelist group",
	*str_common_block =	"a common block",
	*str_dummy =		"a dummy argument",
	*str_result_var =	"the function result variable",

	*str_in_common =	"in a common block",
	*str_equiv =		"equivalence",
	*str_in_namelist =	"in a namelist group",
	*str_init =		"statically initialise",

	*str_alloc_attr =	"ALLOCATABLE",
	*str_external_attr =	"EXTERNAL",
	*str_intent_attr =	"INTENT",
	*str_intrinsic_attr=	"INTRINSIC",
	*str_optional_attr =	"OPTIONAL",
	*str_parameter_attr=	"PARAMETER",
	*str_pointer_attr =	"POINTER",
	*str_private_attr =	"PRIVATE",
	*str_public_attr =	"PUBLIC",
	*str_save_attr =	"SAVE",
	*str_target_attr =	"TARGET",

	*str_const_shape = 	"a constant shape",
	*str_const_def_shape = 	"a constant or deferred shape",
	*str_deferred_shape = 	"deferred shape (i.e. each dimension must be specified as \':\')",
	*str_expl_shape = 	"an explicit shape";
/*}}}*/

/*{{{  int32  build_attrs (int32  attr, old_attrs,  Array *arp) -- VERY GOOD!!*/
int32  build_attrs (attr, old_attrs, arp)
/*-------------------------------------*/
	int32  attr, old_attrs;
	Array  *arp;
/*----------------------------------------------------------------
	Called when parsing a 'typedecl', to add another attribute
('attr') to 'old_attrs'.  'attr' can be:

	AT_ALLOC,  AT_EXTERNAL,  AT_IN,  AT_OUT,  AT_INOUT,
	AT_INTRINSIC,  AT_OPTIONAL,  AT_PARAM,  AT_POINTER,
	AT_PRIVATE,  AT_PUBLIC,  AT_SAVE,  AT_TARGET.

	'arp' contains the shape information provided by a
DIMENSION attribute in the typedecl, if there is one.  If the
shape is inappropriate for the new 'attr' an errmsg is given
(unless the shape is illegal, in which case an error has already
been reported), but it doesn't prevent the new 'attr' from being
set.
	The new value of 'old_attrs' is returned.

N.B. Unlike 'install_attr ()', this doesn't allow for any
'tolerable_attrs', so 'attr' isn't set if it clashes with *any*
existing flag in 'old_attrs'.
----------------------------------------------------------------*/
{
	char  *str_array,  *str_new_attr;
	int32  compatible_attrs;
	int    compatible_shape;
	
	/*------------------------------------------------------*/
	/* If 'old_attrs' and 'arp' are 0, then 'attr' is the	*/
	/* first attribute in the attribute list.  It must be	*/
	/* be legal and so can be installed without checking!	*/
	/*------------------------------------------------------*/
	if ( !(old_attrs || arp))
		return  attr;

	switch (attr) {
	/*{{{  AT_ALLOC*/
	case  AT_ALLOC :
	/*------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access', SAVE,	*/
		/* TARGET and a deferred shape DIMENSION.		*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_SAVE | AT_TARGET);
		compatible_shape = (DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_new_attr = str_alloc_attr;
		str_array    = "ALLOCATABLE arrays";
	
		break;
	/*}}}*/
	/*{{{  AT_EXTERNAL*/
	case  AT_EXTERNAL :
	/*---------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access' and	*/
		/* OPTIONAL.  No DIMENSION information is allowed.	*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_OPTIONAL);
		compatible_shape = SHAPE_MASK;	 /* ...but, if this 'attr' is accepted,
						    a special errmsg given as *no*
						    array-spec is allowed! */
		str_new_attr = str_external_attr;
	
		break;
	/*}}}*/
	/*{{{  AT_IN,  AT_OUT,  AT_INOUT*/
	case  AT_IN :
	case  AT_OUT :
	case  AT_INOUT :
	/*------------*/
		/*------------------------------------------------------*/
		/* Only compatible attributes are OPTIONAL, TARGET and	*/
		/* *any* shape.						*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_OPTIONAL | AT_TARGET);
		compatible_shape = SHAPE_MASK;		/* i.e. any shape is OK! */
	
		str_new_attr = str_intent_attr;
	
		break;
	/*}}}*/
	/*{{{  AT_INTRINSIC*/
	case  AT_INTRINSIC :
	/*----------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access'.  No	*/
		/* DIMENSION information is allowed.			*/
		/*------------------------------------------------------*/
		compatible_attrs = AT_ACCESS;
		compatible_shape = SHAPE_MASK;	 /* ...but, if this 'attr' is accepted,
						    a special errmsg given as *no*
						    array-spec is allowed! */
		str_new_attr = str_intrinsic_attr;
	
		break;
	/*}}}*/
	/*{{{  AT_OPTIONAL*/
	case  AT_OPTIONAL :
	/*---------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are EXTERNAL, INTENT,	*/
		/* POINTER, TARGET and *any* shape.			*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_EXTERNAL | AT_INTENT | AT_POINTER | AT_TARGET);
		compatible_shape = SHAPE_MASK;		/* i.e. any shape is OK! */
	
		str_new_attr = str_optional_attr;
	
		break;
	/*}}}*/
	/*{{{  AT_PARAM*/
	case  AT_PARAM :
	/*------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access', SAVE	*/
		/* and a constant shape DIMENSION.			*/
		/*							*/
		/* N.B. It seems strange that one can specify SAVE for	*/
		/* a named constant, but there doesn't appear to be	*/
		/* anything in the F90 std to disallow it!		*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_SAVE);
		compatible_shape = (CONST_SHAPE | INVALID_SHAPE);
	
		str_new_attr = str_parameter_attr;
		str_array    = "PARAMETER arrays";
	
		break;
	/*}}}*/
	/*{{{  AT_POINTER*/
	case  AT_POINTER :
	/*--------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access',		*/
		/* OPTIONAL, SAVE and a deferred shape DIMENSION.	*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_OPTIONAL | AT_SAVE);
		compatible_shape = (DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_new_attr = str_pointer_attr;
		str_array    = "POINTER arrays";
	
		break;
	/*}}}*/
	/*{{{  AT_PRIVATE,  AT_PUBLIC*/
	case  AT_PRIVATE :
	case  AT_PUBLIC :
	/*--------------*/
		/*------------------------------------------------------*/
		/* The only *incompatible* attributes are 'access',	*/
		/* INTENT, OPTIONAL and an assumed size / shape (as	*/
		/* opposed to deferred shape) DIMENSION.		*/
		/*	However, don't complain about an assumed size /	*/
		/* shape DIMENSION until we know what kind of object is	*/
		/* being declared (as a more specific errmsg can be	*/
		/* given then). 					*/
		/*------------------------------------------------------*/
		compatible_attrs = ~(AT_ACCESS | AT_INTENT | AT_OPTIONAL);
		compatible_shape = SHAPE_MASK;		/* i.e. any shape is OK! */
	
		str_new_attr = ((attr == AT_PRIVATE) ?
		                               str_private_attr : str_public_attr);
	
		break;
	/*}}}*/
	/*{{{  AT_SAVE*/
	case  AT_SAVE :
	/*-----------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access',		*/
		/* ALLOCATABLE, PARAMETER, POINTER, TARGET and a	*/
		/* constant or deferred shape DIMENSION.		*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_ALLOC | AT_PARAM | AT_POINTER |
		                    AT_TARGET);
		compatible_shape = (CONST_SHAPE | DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_new_attr = str_save_attr;
		str_array    = "SAVEd arrays";
	
		break;
	/*}}}*/
	/*{{{  AT_TARGET*/
	case  AT_TARGET :
	/*-------------*/
		/*------------------------------------------------------*/
		/* The only compatible attributes are 'access',		*/
		/* ALLOCATABLE, INTENT, OPTIONAL, SAVE and *any* shape.	*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_ACCESS | AT_ALLOC | AT_INTENT | AT_OPTIONAL |
		                    AT_SAVE);
		compatible_shape = SHAPE_MASK;		/* i.e. any shape is OK! */
	
		str_new_attr = str_target_attr;
	
		break;
	/*}}}*/
	}
	/*{{{  do checking*/
	if (only_flags_set (old_attrs, compatible_attrs)) {
		/*------------------------------------------------------*/
		/* No incompatible 'attr' flags.  If there's a shape	*/
		/* error, report it but still set the 'attr' flag.	*/
		/*------------------------------------------------------*/
		if (arp) {
			if (attr == AT_EXTERNAL)
				errmsg ("array dimensions for EXTERNAL functions can only be specified in an explicit interface");
	
			else if (attr == AT_INTRINSIC)
				errmsg ("array dimensions must not be specified for INTRINSIC functions");
	
			else if ( ! (arp->attrs & compatible_shape))
				shape_errmsg (NULL_Sym, str_array, compatible_shape);
		}
		return  (old_attrs | attr);
	}
	
	/*--------------------------------------------------------------*/
	/* There are incompatible flags in 'old_attrs'.  Report an	*/
	/* error and just return 'old_attrs' without setting new flag.	*/
	/*--------------------------------------------------------------*/
	if (attr & old_attrs)
		errmsg ("%s attribute is duplicated", str_new_attr);
	
	else if ((attr & AT_INTENT)  &&  (old_attrs & AT_INTENT))
		errmsg ("INTENT attribute is specified more than once");
	
	else {
		int32  incompat_attrs = old_attrs & ~compatible_attrs;
	
		if      (incompat_attrs & AT_ALLOC)	begin_errmsg (str_alloc_attr);
		else if (incompat_attrs & AT_EXTERNAL)	begin_errmsg (str_external_attr);
		else if (incompat_attrs & AT_INTENT)	begin_errmsg (str_intent_attr);
		else if (incompat_attrs & AT_INTRINSIC)	begin_errmsg (str_intrinsic_attr);
		else if (incompat_attrs & AT_OPTIONAL)	begin_errmsg (str_optional_attr);
		else if (incompat_attrs & AT_PARAM)	begin_errmsg (str_parameter_attr);
		else if (incompat_attrs & AT_POINTER)	begin_errmsg (str_pointer_attr);
		else if (incompat_attrs & AT_PRIVATE)	begin_errmsg (str_private_attr);
		else if (incompat_attrs & AT_PUBLIC)	begin_errmsg (str_public_attr);
		else if (incompat_attrs & AT_SAVE)	begin_errmsg (str_save_attr);
		else if (incompat_attrs & AT_TARGET)	begin_errmsg (str_target_attr);
	
		end_errmsg (" and %s attributes are incompatible", str_new_attr);
	}
	return  old_attrs;
	/*}}}*/
}
/*}}}*/
/*{{{  install_typedecl (sp, type, type_sp, attrs) -- OK*/
install_typedecl (sp, type, type_sp, attrs)
/*---------------------------------------*/
	Symbol  *sp;
	int32    type;
	Symbol  *type_sp;	/* for TY_DERIVED, the derived-type name */
	int32    attrs;
/*--------------------------------------------------------------*/
/*	Called by 'yyparse()' to install in symbol '*sp' the	*/
/* 'type' and 'attrs' information declared in a typedecl.	*/
/*--------------------------------------------------------------*/
{
	if (undeclared_id (sp)) {
	        /*{{{  nothing declared for ID yet*/
	        /*--------------------------------------------------------------*/
	        /* ID not previously declared.  Note that this means it *isn't*	*/
	        /* a dummy argument.						*/
	        /*--------------------------------------------------------------*/
	        
	        if (attrs & (AT_INTENT | AT_OPTIONAL)) {
	        	/*{{{  INTENT / OPTIONAL illegal for non-dummy; give errmsg & clear flags*/
	        	/*--------------------------------------------------------------*/
	        	/* INTENT and OPTIONAL are illegal for a non dummy argument.	*/
	        	/* Give an errmsg and unset the flags.				*/
	        	/*--------------------------------------------------------------*/
	        	if ((attrs & AT_INTENT)  &&  (attrs & AT_OPTIONAL))
	        		begin_errmsg ("INTENT and OPTIONAL attributes");
	        	
	        	else if (attrs & AT_INTENT)
	        		begin_errmsg ("INTENT");
	        	
	        	else
	        		begin_errmsg ("OPTIONAL attribute");
	        	
	        	end_errmsg (" specified for non dummy argument %q", sp->name);
	        	clear_flags (attrs, (AT_INTENT | AT_OPTIONAL));
	        	/*}}}*/
	        }
	        else if (attrs & AT_INTRINSIC) {
	        	/*{{{  if not found in 'intrintab', give errmsg & clear flag*/
	        	/*--------------------------------------------------------------*/
	        	/* Intrinsic procedure name should be found in 'intrintab'.	*/
	        	/* If not, clear the AT_INTRINSIC flag.				*/
	        	/*								*/
	        	/* N.B. If either AT_INTENT or AT_OPTIONAL is set, AT_INTRINSIC	*/
	        	/* won't be -- hence the 'else if'.				*/
	        	/*--------------------------------------------------------------*/
	        	Symbol  *intrin_sp;
	        	
	        	if (intrin_sp = lookup_intrin_proc (sp->name)) {
	        		sp->info.sp = intrin_sp;
	        		attrs |= ( AT_INFO |
	        		          ((intrin_sp->attrs & AT_RESULT_VAR) ? AT_FUNC : AT_SUBR) );
	        	
	        		if (attrs & AT_SUBR) {
	        			errmsg ("intrinsic subroutine %q has been given a data type",
	        			         sp->name);
	        			type = 0;
	        		}
	        		else if ( ! (type & intrin_fn_result_types (intrin_sp)) ) {
	        			errmsg ("intrinsic function %q incorrectly typed", sp->name);
	        			type = 0;
	        		}
	        	}
	        	else {
	        		errmsg ("unknown intrinsic procedure: %q", sp->name);
	        		clear_flags (attrs, AT_INTRINSIC);
	        	}
	        	/*}}}*/
	        }
	        
	        if (attrs & AT_EXTERNAL) {
	        	/*{{{  an external (non-dummy) function -- try to install in 'global_list'*/
	        	/*---------------------------------------------------------------
	        		Declared as an EXTERNAL (non-dummy) function, so call
	        	'install_global_entity (sp, AT_EXTERNAL)' to try to install it
	        	in the 'global_list'.
	        		If 'install_global_entity ()' returns TRUE, it means that
	        	the EXTERNAL declaration doesn't conflict with a previous global
	        	use of 'sp->name'.  In this case the function installs it in the
	        	'global_list', sets its AT_EXTERNAL flag, and also inherits and
	        	sets its 'proc-type' (AT_FUNC or AT_SUBR) if it was determined
	        	by any previous use.  If AT_SUBR is thus set, give an errmsg
	        	since it can't be given a datatype.  If the 'proc-type' *isn't*
	        	determined by a previous use, call 'set_external_proc_type ()'
	        	to set AT_FUNC in 'sp' and all equivalent entries in the 'global_list',
	        	since 'sp's appearance in a type-decl implies that it's a function.
	        		If 'install_global_entity ()' returns FALSE, this means
	        	that 'sp->name' has already been used for a different type of
	        	global entity (presumably in a previous scope unit!), so it
	        	doesn't install AT_EXTERNAL or put it in the 'global_list'.
	        	In this case we treat it as a local function, so we install
	        	AT_FUNC and the 'type'.
	        	---------------------------------------------------------------*/
	        	
	        	if (install_global_entity (sp, AT_EXTERNAL)) {
	        		/*------------------------------------------------------*/
	        		/* Accepted as an external procedure.  AT_EXTERNAL has	*/
	        		/* been set, and AT_FUNC or AT_SUBR were inherited if	*/
	        		/* set in a previous scope unit.			*/
	        		/*------------------------------------------------------*/
	        		if (sp->attrs & AT_SUBR) {
	        			/*----------------------------------------------*/
	        			/* an external subroutine, so give an errmsg	*/
	        			/* and ignore type and DIMENSION info.  	*/
	        			/*----------------------------------------------*/
	        			errmsg ("external subroutine %q given a data type", sp->name);
	        			type = 0;
	        		}
	        		else if ( ! (sp->attrs & AT_FUNC) )
	        			/*----------------------------------------------*/
	        			/* set AT_FUNC (since it's in a typedecl), and	*/
	        			/* propagate it through the 'global_list'.	*/
	        			/*----------------------------------------------*/
	        			set_external_proc_type (sp, AT_FUNC);
	        	}
	        	
	        	/*--------------------------------------------------------------*/
	        	/* ...otherwise 'sp->name' was used as a different kind of	*/
	        	/* global entity in a previous scope unit, and an appropriate	*/
	        	/* errmsg is given by 'install_global_entity()'.  Therefore	*/
	        	/* ignore the 'EXTERNAL' attr.					*/
	        	/*--------------------------------------------------------------*/
	        	attrs = sp->attrs;	/* ...as appropriate flags already set in
	        				   'sp->attrs' by above func call, but
	        				   'attrs' is assigned to it below! */
	        	/*}}}*/
	        }
	        
	        sp->type = type;
	        if (type & TY_DERIVED)
	        	sp->typeinfo.type_sp = type_sp;
	        sp->attrs = attrs;
	        /*}}}*/
	}
	else {
		/*{{{  ID already declared*/
		/*------------------------------*/
		/*  ID is already declared	*/
		/*------------------------------*/
		if (sp->attrs & AT_UNTYPED_ENTITY) {
			/*{{{  Can't be typed.  Give errmsg & return or override the entity type*/
			/*--------------------------------------------------------------*/
			/* An object that can't have a data type.  Report an error, and	*/
			/* then do one of two things:					*/
			/*								*/
			/* -- if the object type is one of those in AT_OVERRIDE_ATTRS	*/
			/* (i.e. AT_CONSTRUCT), then clear this flag and set the data	*/
			/* type.  In other words, forget the object's old identity and	*/
			/* give it a new one!						*/
			/*								*/
			/* -- otherwise return.						*/
			/*--------------------------------------------------------------*/
			switch (sp->attrs & AT_UNTYPED_ENTITY) {
			case  AT_SUBR :			begin_errmsg ("subroutine");	break;
			case  AT_MODULE :		begin_errmsg ("module");	break;
			case  AT_PROGRAM :		begin_errmsg ("main program");	break;
			case  AT_BLOCK_DATA :		begin_errmsg ("block data name");  break;
			case  AT_DERIVED_TYPE :		begin_errmsg ("derived type");  break;
			case  AT_NAMELIST_GROUP :	begin_errmsg ("namelist group");  break;
			case  AT_CONSTRUCT :		begin_errmsg ("construct name");  break;
			case  AT_PROCESSORS :		begin_errmsg ("processors arrangement");  break;
			case  AT_TEMPLATE :		begin_errmsg ("template");	break;
			}
			end_errmsg (" %q cannot have a datatype", sp->name);
			
			if (sp->attrs & AT_CONSTRUCT) {
				clear_flags (sp->attrs, AT_CONSTRUCT);
				sp->type |= type;
				if (type & TY_DERIVED)
					sp->typeinfo.type_sp = type_sp;
			}
			else
				return;
			/*}}}*/
		}
		else if (sp->type) {
			/*{{{  already typed, implicitly or explicitly.  Check for consistency*/
			/*--------------------------------------------------------------*/
			/* Already typed, implicitly or explicitly.			*/
			/* If it's been implicitly typed and the explicit type agrees,	*/
			/* unset the TY_IMPLICIT flag.  Otherwise report an error.	*/
			/*--------------------------------------------------------------*/
			if ( !(sp->type & TY_IMPLICIT))
				errmsg ("type already declared for %q", sp->name);
			
			else if (sp->type != type  ||  sp->typeinfo.type_sp != type_sp) {
				begin_errmsg ("type declared for %q disagrees with that assumed in a previous ",
				               sp->name);
				if (sp->attrs & (AT_PARAM | AT_IN_NAMELIST))
					end_errmsg ("%s statement",
					        (sp->attrs & AT_PARAM) ? "PARAMETER" : "NAMELIST");
				else
					end_errmsg ("reference");
			}
			else
				clear_flags (sp->type, TY_IMPLICIT);
			/*}}}*/
		}
		else if (sp->attrs & AT_FUNC) {
			/*{{{  set type if not host func, & it agrees with interface, if any*/
			/*----------------------------------------------------------------
				'sp->attrs' has the AT_FUNC flag set.
				If 'sp' is the host procedure, then the fact that AT_FUNC
			rather than AT_RESULT_VAR is set means that there's a separate
			result variable.  In this case all declarations must refer to the
			latter, not to function name 'sp', so give an errmsg and return
			(i.e. don't install 'attrs' either!)
				If 'sp' has an explicit or intrinsic interface,	check
			for type consistency.
				If AOK, install the type.
			----------------------------------------------------------------*/
			if (sp == this_unitp->sp) {
				/*------------------------------------------------------*/
				/* 'sp' is the host's function name (not result var!)	*/
				/*------------------------------------------------------*/
				Symbol  *result_sp;
			
				for (result_sp = sp->nextid;  ! (result_sp->attrs & AT_RESULT_VAR);
				     result_sp = result_sp->nextid)
					;
				errmsg ("declarations must apply to the result variable %q, not to the function name %q",
				         result_sp->name,  sp->name);
				return;
			}
			else if (sp->attrs & AT_INFO) {
				/*------------------------------------------------------*/
				/* 'sp' is an external or intrinsic function with an	*/
				/* explicit interface.  Check that the declared type	*/
				/* matches that in the interface (which must be set!).	*/
				/*							*/
				/* N.B. The AT_INFO field can't refer to the definition	*/
				/* of an internal or module function, as its data type	*/
				/* can't be installed *after* its definition!		*/
				/*------------------------------------------------------*/
				Symbol  *func_sp = sp->info.sp;		/* the explicit interface */
			
				if (func_sp->attrs & AT_INTRINSIC) {
					if ( !(type & intrin_fn_result_types (func_sp)) ) {
						errmsg ("intrinsic function %q incorrectly typed",
						         sp->name);
						type = 0;
					}
				}
				else if (type != (func_sp->type & TY_TYPE)  ||
				                         type_sp != func_sp->typeinfo.type_sp) {
					errmsg ("type declared for function %q disagrees with its interface body",
					         sp->name);
					type = (func_sp->type & TY_TYPE);	/* correct it! */
					if (type & TY_DERIVED)
						type_sp = func_sp->typeinfo.type_sp;
				}
			}
			sp->type |= type;
			if (type & TY_DERIVED)
				sp->typeinfo.type_sp = type_sp;
			
			/*}}}*/
		}
		else {
			/*{{{  install type.  For a func, check with interface if any*/
			/*--------------------------------------------------------------*/
			/* OK -- typeable and not previously typed, so install the type.*/
			/*--------------------------------------------------------------*/
			
			if (sp->attrs & AT_EXTERNAL) {
				/*------------------------------------------------------*/
				/* 'sp' was declared EXTERNAL and must be a function as	*/
				/* it's typed, so install the AT_FUNC flag and, for non	*/
				/* dummy proc, propagate it through the 'global_list'.	*/
				/*							*/
				/* N.B. We know that AT_SUBR isn't installed as that	*/
				/* has already been checked for.			*/
				/*------------------------------------------------------*/
				if (sp->attrs & AT_DUMMY)
					sp->attrs |= AT_FUNC;
				else
					set_external_proc_type (sp, AT_FUNC);
			}
			
			sp->type |= type;
			if (type & TY_DERIVED)
				sp->typeinfo.type_sp = type_sp;
			/*}}}*/
		}
		
		/*--------------------------------------------------------------*/
		/* install entity type, which may be *just one* one of		*/
		/* AT_ALLOC, AT_POINTER, AT_PARAM or 0.				*/
		/*--------------------------------------------------------------*/
		if ((attrs & AT_ENTITY) && ! install_attr (sp, (attrs & AT_ENTITY)))
			return;
		
		/*--------------------------------------------------------------*/
		/* install attrs, which can be one or more of:			*/
		/* AT_ACCESS, AT_EXTERNAL, AT_INTENT, AT_INTRINSIC, AT_OPTIONAL,*/
		/* AT_SAVE, AT_TARGET.						*/
		/*--------------------------------------------------------------*/
		if (attrs & AT_ATTRS) {
			if ((attrs & AT_ACCESS) && ! install_attr (sp, (attrs & AT_ACCESS)))
				return;
		
			/*--------------------------------------------------*/
			/* the following are 3 mutually exclusive groups... */
			/*--------------------------------------------------*/
			if (attrs & AT_INTRINSIC) {
				if ( ! install_attr (sp, AT_INTRINSIC)) return;
			}
			else if (attrs & AT_SAVE) {
				if ( ! install_attr (sp, AT_SAVE)) return;
			}
			else {
				if ((attrs & AT_EXTERNAL) && ! install_attr (sp, AT_EXTERNAL))
					return;
		
				if ((attrs & AT_INTENT) && ! install_attr (sp, (attrs & AT_INTENT)))
					return;
		
				if ((attrs & AT_OPTIONAL) && ! install_attr (sp, AT_OPTIONAL))
					return;
			}
			if ((attrs & AT_TARGET) && ! install_attr (sp, AT_TARGET))
				return;
		}
		
		/*}}}*/
	}
}
/*}}}*/

/******************************
-- 'install_global_entity()' SHOULD INSTALL / CHECK / PROPAGATE arp (& type)
AS WELL!  LOOK INTO THIS!

-- SET previous_error (OR WHATEVER CALLED) WHEN APPROPRIATE!

-- WHEN INSTALL char_len, WHAT ABOUT CLASHES WITH PREV DECL.
*******************************/

/*{{{  bool  install_attr (sp, attr) -- OK*/
bool  install_attr (sp, attr)
/*-------------------------*/
	Symbol  *sp;
	int32  attr;
{
	bool   align_forbidden,  distr_forbidden,  first_try,  set_type,
	       type_set_for_nondata;
	char  *str_array,  *str_attr;
	int32  compatible_attrs,  tolerable_attrs;
	int    compatible_shape;
	
/*{{{  *** STILL TO DO ****/
/*--------------------------------------------------
-- Look at whather want to override 'AT_CONSTRUCT' -- probably not worth
	making an exception of this case!

-- Look into over-riding certain attrs, e.g. AT_SAVE, AT_TARGET
	-- basically, whenever errmsg suggests that an new entity-type
	overrides a previously declared attr, e.g
	"cannot SAVE an external procedure"

-- If we do the above, generate errmsgs for everything over-ridden?

-- In 'attr_errmsg', set 'earlier_error' where appropriate (i.e
	where we're complaining about an attr that was set previously,
	as in the above SAVE example).

--------------------------------------------------*/
/*}}}*/
/*{{{  Notes*/
#if 0

--  'attr' can be any attr flag *except*:
	AT_STMT_VAR,  AT_PROGRAM,  AT_BLOCK_DATA,  AT_RESULT_VAR,
	AT_DUMMY,  AT_NORMAL_VAR.    (See below for the last of these).

--  Never reject anything because of an:
	-- AT_STMT_VAR flag, as this can coexist with *anything*,
	      as it's a stmt entity which can simulateeously be used
	      for any local/global entity.
	-- AT_INFO flag, because it's always accompanied by an entity
	      flag (to determine the contents of the 'info' field), and
	      the latter is used to determine whether the new flag is OK!

--  When this is called we don't expect AT_STMT_VAR to be set, because
that flag is only set immediately before parsing expressions and will
be unset immediately afterwards (*** unless there's a syntax error! ***).
During the parsing of such exprs, 'install_attr ()' isn't called because
the actions for 'part_ref' won't call 'install_attr ()' if AT_STMT_VAR
is set.
	However, we include AT_STMT_VAR in 'compatible_attrs', in case
it *is* set as a result of an earlier syntax error occurring before
it could be unset!

--  We unset AT_MODULE, AT_PROGRAM, AT_BLOCK_DATA or AT_CONSTRUCT if they're
set and clash with another 'entity-attr' to be installed, as they're not
used in main body of prog unit.

-- The attrs SAVE & TARGET don't prevent the setting of an
entity flag (e.g. INTRINSIC, EXTERNAL, AUTO, PARAM, DERIVED_TYPE, etc).
We just assume that the attr is inappropriate.

/*{{{  Setting of AT_NORMAL_VAR*/
Setting of AT_NORMAL_VAR
~~~~~~~~~~~~~~~~~~~~~~~~
	This function is never called to install 'AT_NORMAL_VAR' directly
(though it may set it as a side-effect of setting AT_INIT).
	It's set in the following circumstances:

-- by initialisation:  set as a side-effect of 'install_attr (sp, AT_INIT)',
	called by 'yyparse()' for an initialisation in a 'type-decl'
	or 'data_stmt';

-- by reference:  AT_NORMAL_VAR is set by 'yyparse()' in rules
	'part_ref', 'do_var' or 'assign_var';

-- by an explicit interface body:  at the end of the interface body,
	'process_proc_interface()' sets 'AT_NORMAL_VAR' wherever
	possible for ID's that have AT_RESULT_VAR or AT_DUMMY set.

Compatible 'sp->attrs' flags for AT_NORMAL_VAR
----------------------------------------------
For reference, compatible 'sp->attrs' flags are:

ok  = compatible;
x   = incompatible, and don't set new flag;
--x = incompatible, but still set new flag.
o   = incompatible, but replace old flag by new.

	AT_ALLOC		x
	AT_POINTER		x
	AT_NORMAL_VAR		ok
	AT_STMT_VAR		ok
	
	AT_PARAM		x

	AT_FUNC			x
	AT_SUBR			x
	AT_MODULE		x
	AT_PROGRAM		x
	AT_BLOCK_DATA		x

	AT_DERIVED_TYPE		x
	AT_NAMELIST_GROUP	x
	AT_CONSTRUCT		o
	AT_PROCESSORS		x
	AT_TEMPLATE		x

	AT_COMMON_BLOCK		} ok
	AT_COMMON_SAVE		}

	AT_DUMMY		ok
	AT_RESULT_VAR		ok
	AT_IN_COMMON		ok
	AT_EQUIV		ok

	AT_EXTERNAL		x
	AT_INTRINSIC		x
	AT_IN			ok
	AT_OUT			ok
	AT_OPTIONAL		ok
	AT_PRIVATE		ok
	AT_PUBLIC		ok
	AT_SAVE			ok
	AT_TARGET		ok

	AT_IN_NAMELIST		ok
	AT_INFO			ok

/*}}}*/

#endif

/************************************************
OVERRIDE_ATTRS (e.g. SAVE, TARGET) ??
Problem is:
	-- maybe there's also a conflict with something more important,
so new_attr is ignored; then inappropriate to remove SAVE, etc.
E.g.:
	POINTER P
	SAVE P
	ALLOCATABLE P -- can't be, as already a ptr; don't unset SAVE.

Ah, but only overridden if *all* of incompatible_attrs can be ignored.
************************************************/
/*}}}*/

	if (undeclared_id (sp)) {
		/*{{{  simplified checking*/
		/*----------------------------------------------*/
		/* First appearance of name 'sp->name'...	*/
		/*----------------------------------------------*/
		switch (attr) {
		case AT_IN :
		case AT_OUT :
		case AT_INOUT :
			errmsg ("INTENT specified for non dummy argument %q", sp->name);
			return  FALSE;
		
		case AT_OPTIONAL :
			errmsg ("OPTIONAL attribute specified for non dummy argument %q", sp->name);
			return  FALSE;
		
		case AT_INTRINSIC :
			if (sp->info.sp = lookup_intrin_proc (sp->name)) {
				sp->attrs = (AT_INTRINSIC | AT_INFO |
				             ((sp->info.sp->attrs & AT_RESULT_VAR) ? AT_FUNC : AT_SUBR));
				return  TRUE;
			}
			else {
				errmsg ("unknown intrinsic procedure: %q", sp->name);
				return  FALSE;
			}
		
		case AT_MODULE :
		case AT_COMMON_BLOCK :
		case AT_EXTERNAL :
			return   install_global_entity (sp, attr);
		
		case AT_COMMON_SAVE :
			if (install_global_entity (sp, AT_COMMON_BLOCK)) {
				sp->attrs |= AT_COMMON_SAVE;
				return  TRUE;
			}
			else
				return FALSE;
				
		case  AT_INIT :
			sp->attrs = (AT_INIT | AT_NORMAL_VAR);
			lookup_or_install_type (sp);
			return  TRUE;
		
		case AT_PARAM :
		case AT_IN_NAMELIST :
			lookup_or_install_type (sp);	/* set implicit type */
			
			/* ...now fall throught to 'default' case to set flag	*/
		
		default :
			sp->attrs = attr;
			return  TRUE;
		}
		/*}}}*/
	}
	set_type = type_set_for_nondata = align_forbidden =
	                                  distr_forbidden = FALSE;
	first_try = TRUE;
	switch (attr) {
	/*{{{  case  AT_ALLOC :*/
	case  AT_ALLOC :
	/*------------*/
		/*{{{  COMMENT: compatible attrs*/
		/*------------------------------------------------------
		ALLOCATABLE: compatible 'attr' flags
		------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
			
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK } ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	--x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	--x
			AT_INFO		ok
		------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_STMT_VAR | AT_COMMON_NAME | AT_ACCESS |
		                    AT_SAVE | AT_TARGET | AT_INFO);
	
		tolerable_attrs  = (AT_CONSTRUCT | AT_IN_COMMON | AT_IN_NAMELIST);
	
		compatible_shape = (DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_alloc;
		str_array = "allocatable array";
	
		break;
	/*}}}*/
	/*{{{  case  AT_POINTER :*/
	case  AT_POINTER :
	/*--------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*------------------------------------------------------
		POINTER: compatible 'attr' flags
		--------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	ok
			AT_RESULT_VAR	ok
			AT_IN_COMMON	ok
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		--x
			AT_OUT		--x
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	--x
		
			AT_IN_NAMELIST	--x
			AT_INFO		ok
		------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_STMT_VAR | AT_COMMON_NAME | AT_DUMMY |
		                    AT_RESULT_VAR | AT_IN_COMMON | AT_OPTIONAL |
		                    AT_ACCESS | AT_SAVE | AT_INFO);
		
		tolerable_attrs  = (AT_CONSTRUCT | AT_INTENT | AT_TARGET |
		                    AT_IN_NAMELIST);
	
		compatible_shape = (DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_pointer;
		str_array = "pointer array";
	
		break;
	/*}}}*/
	/*{{{  case  AT_PARAM : -- sets type if not yet declared*/
	case  AT_PARAM :
	/*------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*------------------------------------------------------
		PARAMETER: compatible 'attr' flags
		----------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} x
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	--x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	--x
		
			AT_IN_NAMELIST	--x
			AT_INFO		ok
		------------------------------------------------------*/
		/*}}}*/
	
		/*------------------------------------------------------*/
		/* N.B. A side-effect of installing this flag is that	*/
		/* an implicit data-type is installed if the data-type	*/
		/* isn't already declared.				*/
		/*------------------------------------------------------*/
		
		compatible_attrs = (AT_STMT_VAR | AT_ACCESS | AT_SAVE | AT_INFO);
		
		tolerable_attrs  = (AT_CONSTRUCT | AT_IN_COMMON | AT_TARGET |
		                    AT_IN_NAMELIST);
	
		compatible_shape = (CONST_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_param;
		str_array = "named constant array";
	
		set_type = TRUE;
		break;
	/*}}}*/
	/*{{{  case  AT_FUNC :*/
	case  AT_FUNC :
	/*-----------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		FUNCTION (*not* a result variable):  compatible 'attr' flags
		------------------------------------------------------------
		This is only invoked as a result of an explicit interface or a
		function reference.  In a 'func-stmt', AT_RESULT_VAR (& possibly
		AT_FUNC) are installed without checking, as a new name is declared!
		
		N.B.  'AT_FUNC' is disjoint from 'AT_RESULT_VAR'.  The former means
		it's used as a function.  The latter means its used as a variable,
		which can be an 'AT_POINTER' or 'AT_NORMAL_VAR'.  E.g.:
		
			A = F(2.0)			! F == function;
			FUNCTION F (X)			! F == result var (can't reference
							       F as a function)
			FUNCTION F (X), RESULT (G)	! F == function;
							! G == result var.
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		ok -- installed by ref or interface
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} x  (if an external function)
			AT_COMMON_SAVE	} ok (if an internal/module/stmt func)
		
			AT_DUMMY	ok
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	ok
			AT_INTRINSIC	ok
			AT_IN		--x
			AT_OUT		--x
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_STMT_VAR | AT_FUNC | AT_COMMON_NAME |
		                    AT_DUMMY | AT_EXTERNAL | AT_INTRINSIC |
		                    AT_OPTIONAL | AT_ACCESS | AT_INFO);
	try_to_set_func :
	
		if (only_flags_set (sp->attrs, compatible_attrs)) {
			/*------------------------------------------------------*/
			/* If AT_FUNC isn't already set, and it's a non-dummy	*/
			/* external proc, call 'set_external_proc_type()' to	*/
			/* set AT_FUNC in every entry for this 'name' in the	*/
			/* 'global_list'!					*/
			/*	If a shape is specified, report an error	*/
			/* (provided that neither AT_FUNC nor AT_EXTERNAL are	*/
			/* set, in which case the error would have been reported*/
			/* already) but still set the flags as before.		*/
			/*------------------------------------------------------*/
			if (sp->arp  &&  ! (sp->attrs & (AT_FUNC | AT_EXTERNAL)) )
				errmsg ("array dimensions for function %q can only be specified in an explicit interface or function definition",
				         sp->name);
	
			/*{{{  if 'sp->alignp / distp' set, give errmsg & delete it*/
			if (sp->alignp) {
				earlier_error = TRUE;
				if (sp->alignp->tgt_sp)
					errmsg ("can only specify alignment of function result %q in an explicit interface or function definition",
						 sp->name);
				else
					errmsg ("cannot specify INHERIT for function %q", sp->name);
				delete_align (sp->alignp, DECR_NLINKS);
				sp->alignp = NULL_Align;
				earlier_error = FALSE;
			}
			
			if (sp->distp) {
				earlier_error = TRUE;
				errmsg ("can only specify distribution of function result %q in an explicit interface or function definition",
					 sp->name);
				delete_distr (sp->distp, DECR_NLINKS);
				sp->distp = NULL_Distr;
				earlier_error = FALSE;
			}
			/*}}}*/
	
			if (  (sp->attrs & AT_EXTERNAL)  &&
			    ! (sp->attrs & (AT_DUMMY | AT_FUNC)) )
				set_external_proc_type (sp, AT_FUNC);
			else
				sp->attrs |= AT_FUNC;
	
			if ( ! first_try)
				clear_flags (sp->attrs, AT_CONSTRUCT);
			return  TRUE;
		}
	
		/*---------------------*/
		/* Failed!  Try again? */
		/*---------------------*/
		if (first_try) {
			attr_errmsg (sp, AT_FUNC, str_func,
			                             (sp->attrs & ~compatible_attrs));
			/*------------------------------------------------------*/
			/* the following incompatible flags don't prevent the	*/
			/* new attr being set...				*/
			/*------------------------------------------------------*/
			compatible_attrs |= (AT_CONSTRUCT | AT_INTENT | AT_SAVE |
			                     AT_TARGET);
			first_try = FALSE;
			goto  try_to_set_func;
		}
		else
			return  FALSE;
	/*}}}*/
	/*{{{  case  AT_SUBR :*/
	case  AT_SUBR :
	/*-----------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		SUBROUTINE: compatible 'attr' flags
		-----------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		ok -- installed by ref, interface or def
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} x  (if an external subr)
			AT_COMMON_SAVE	} ok (if an internal/module subr)
		
			AT_DUMMY	ok
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	ok
			AT_INTRINSIC	ok
			AT_IN		--x
			AT_OUT		--x
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok
		
		Also, this 'attr' is incompatible with the presence of a datatype
		(unless its implicitly set for a dummy argument!) or shape,
		so this is also checked for.
		--------------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_STMT_VAR | AT_SUBR | AT_COMMON_NAME |
		                    AT_DUMMY | AT_EXTERNAL | AT_INTRINSIC |
		                    AT_OPTIONAL | AT_ACCESS | AT_INFO);
	
	try_to_set_subr :
		if (only_flags_set (sp->attrs, compatible_attrs)  &&  ! sp->arp) {
	
			/*{{{  if datatype *explicitly* set, give errmsg & return FALSE*/
			/*----------------------------------------------------------------
				If 'sp' is a dummy argument it may have had an implicit
			datatype 'tentatively' set (to determine the prefix for its name).
			In that case, since there are no other 'errors' we simply ignore
			the datatype and proceed.   (Incidentally, we *don't* unset it,
			since that would change 'sp's name prefix and thus its name!).
			
				However, if 'sp' has a type *explicitly* set, or
			implicitly set when it isn't a dummy argument, then give an errmsg
			and return FALSE.
			----------------------------------------------------------------*/
			if (   (sp->type & TY_TYPE)  &&
			    ! ((sp->attrs & AT_DUMMY)  &&  (sp->type & TY_IMPLICIT)) ) {
			
				attr_errmsg (sp, AT_SUBR, str_subr, 0);
				return  FALSE;
			}
			/*}}}*/
	
			/*{{{  if 'sp->alignp / distp' set, give errmsg & delete it*/
			if (sp->alignp) {
				earlier_error = TRUE;
				if (sp->alignp->tgt_sp)
					errmsg ("subroutine %q cannot be aligned", sp->name);
				else
					errmsg ("cannot specify INHERIT for subroutine %q", sp->name);
				delete_align (sp->alignp, DECR_NLINKS);
				sp->alignp = NULL_Align;
				earlier_error = FALSE;
			}
			
			if (sp->distp) {
				earlier_error = TRUE;
				errmsg ("subroutine %q cannot be distributed", sp->name);
				delete_distr (sp->distp, DECR_NLINKS);
				sp->distp = NULL_Distr;
				earlier_error = FALSE;
			}
			/*}}}*/
	
			/*------------------------------------------------------*/
			/* If AT_SUBR isn't already set, and it's a non-dummy	*/
			/* external proc (i.e. its name is a global entity),	*/
			/* call 'set_external_proc_type()' to set AT_SUBR in	*/
			/* every entry for this 'name' in the 'global_list'!	*/
			/*------------------------------------------------------*/
			if (  (sp->attrs & AT_EXTERNAL)  &&
			    ! (sp->attrs & (AT_DUMMY | AT_SUBR)) )
				set_external_proc_type (sp, AT_SUBR);
			else
				sp->attrs |= AT_SUBR;
	
			if ( ! first_try)
				clear_flags (sp->attrs, AT_CONSTRUCT);
			return  TRUE;
		}
	
		/*---------------------*/
		/* Failed!  Try again? */
		/*---------------------*/
		if (first_try) {
			attr_errmsg (sp, AT_SUBR, str_subr,
			                         (sp->attrs & ~compatible_attrs));
			if (sp->arp)
			 	return  FALSE;
	
			/*------------------------------------------------------*/
			/* the following incompatible flags don't prevent the	*/
			/* new attr being set...				*/
			/*------------------------------------------------------*/
			compatible_attrs |= (AT_CONSTRUCT | AT_INTENT | AT_SAVE |
			                     AT_TARGET);
			first_try = FALSE;
			goto  try_to_set_subr;
		}
		else
			return  FALSE;
	/*}}}*/
	/*{{{  case  AT_MODULE : -- called from USE stmt; don't set if incompat*/
	case  AT_MODULE :
	/*-------------*/
		/*------------------------------------------------------
			This is only compatible with itself - AT_MODULE,
		or with AT_STMT_VAR.  (It isn't even compatible with
		'AT_COMMON_BLOCK', as all global names must be distinct).
		It isn't compatible with the presence of a datatype or
		shape.
			We don't set it if another 'attr' flag is
		already set, as it serves no purpose except to check
		for name clashes & recursive module refs (i.e. where
		a module USEs itself), and these purposes are served
		by reporting the error when it's encountered, without
		setting the 'AT_MODULE' flag.  (Incidentally,
		multiple USE of the same module name *isn't* an
		error, but check for this in the caller!).
			If 'sp->attrs' has only compatible flags set,
		and no datatype or shape, 'install_global_entity()'
		is called to check that 'sp' isn't already used for
		another global entity, and that the module doesn't
		USE itself.  If it's ok it sets the 'AT_MODULE' flag.
		It is called even if AT_MODULE is already set, in
		order to check for self-references.
		------------------------------------------------------*/
	
		if (  only_flags_set (sp->attrs, (AT_STMT_VAR | AT_MODULE))  &&
		  ! ((sp->type & TY_TYPE) || sp->arp) ) {
	
			/*{{{  if 'sp->alignp / distp' set, give errmsg & delete it*/
			if (sp->alignp) {
				earlier_error = TRUE;
				if (sp->alignp->tgt_sp)
					errmsg ("module %q cannot be aligned", sp->name);
				else
					errmsg ("cannot specify INHERIT for module %q", sp->name);
				delete_align (sp->alignp, DECR_NLINKS);
				sp->alignp = NULL_Align;
				earlier_error = FALSE;
			}
			
			if (sp->distp) {
				earlier_error = TRUE;
				errmsg ("module %q cannot be distributed", sp->name);
				delete_distr (sp->distp, DECR_NLINKS);
				sp->distp = NULL_Distr;
				earlier_error = FALSE;
			}
			/*}}}*/
	
			return   install_global_entity (sp, AT_MODULE);
		}
	
		attr_errmsg (sp, AT_MODULE, str_module,
		                     (sp->attrs & ~(AT_STMT_VAR | AT_MODULE)));
	
		return  FALSE;
	
		/*------------------------------------------------------*/
		/* N.B. All incompatible flags prevent the new attr	*/
		/* being set!						*/
		/*------------------------------------------------------*/
	/*}}}*/
	/*{{{  case  AT_DERIVED_TYPE :*/
	case  AT_DERIVED_TYPE :
	/*-------------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*------------------------------------------------------
		Derived type: compatible 'attr' flags
		-------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok
		
		Also, this 'attr' is incompatible with the presence
		of a datatype or shape, so this is also checked for.
		------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_STMT_VAR | AT_COMMON_NAME | AT_ACCESS | AT_INFO);
	
		tolerable_attrs  = (AT_CONSTRUCT | AT_SAVE | AT_TARGET);
	
		str_attr = str_derived_type;
	
		type_set_for_nondata = ((sp->type & TY_TYPE) || sp->arp);
		align_forbidden = distr_forbidden = TRUE;
		break;
	/*}}}*/
	/*{{{  case  AT_NAMELIST_GROUP :*/
	case  AT_NAMELIST_GROUP :
	/*---------------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*------------------------------------------------------
		Namelist group: compatible 'attr' flags
		---------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	ok
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok
		
		Also, this 'attr' is incompatible with the presence
		of a datatype or shape, so this is also checked for.
		------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_STMT_VAR | AT_NAMELIST_GROUP | AT_COMMON_NAME |
		                    AT_ACCESS | AT_INFO);
		
		tolerable_attrs  = (AT_CONSTRUCT | AT_SAVE | AT_TARGET);
	
		str_attr = str_namelist_group;
	
		type_set_for_nondata = ((sp->type & TY_TYPE) || sp->arp);
		align_forbidden = distr_forbidden = TRUE;
		break;
	/*}}}*/
	/*{{{  case  AT_CONSTRUCT :*/
	case  AT_CONSTRUCT :
	/*----------------*/
		/*------------------------------------------------------
		Named construct:
		----------------
			Only set during the exec-part!
			It's incompatible with all other attrs except
		for (AT_STMT_VAR | AT_COMMON_BLOCK | AT_COMMON_SAVE), and is
		also incompatible with the presence of a datatype or
		shape.  AT_CONSTRUCT isn't set if there are any
		incompatible attrs, or a datatype or shape, as it
		only serves to check for name clashes and isn't
		otherwise used!
		------------------------------------------------------*/
		compatible_attrs = (AT_STMT_VAR | AT_COMMON_NAME);
	
		tolerable_attrs  = 0;
		
		str_attr = str_construct;
	
		type_set_for_nondata = ((sp->type & TY_TYPE) || sp->arp);
		align_forbidden = distr_forbidden = TRUE;
		break;
	/*}}}*/
	/*{{{  case  AT_PROCESSORS :  case  AT_TEMPLATE :*/
	case  AT_PROCESSORS :
	case  AT_TEMPLATE :
	/*-----------------*/
		/*{{{  COMMENT: compatible attrs*/
		/*------------------------------------------------------
		PROCESSORS / TEMPLATE: compatible 'attr' flags
		----------------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
			
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	--x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok -- ** really should be illegal as HPF objects **
			AT_PUBLIC	ok -- ** should only appear in HPF directives! **
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	--x
			AT_INFO		ok -- not ok, but will conflict with another flag!
		------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_STMT_VAR | AT_COMMON_NAME | AT_ACCESS | AT_INFO);
	
		tolerable_attrs  = (AT_CONSTRUCT | AT_IN_COMMON | AT_SAVE |
		                    AT_TARGET | AT_IN_NAMELIST);
	
		compatible_shape = (EXPL_SHAPE | INVALID_SHAPE);
	
		if (attr == AT_PROCESSORS) {
			str_attr  = str_processors;
			str_array = "processor array";
			align_forbidden = distr_forbidden = TRUE;
		}
		else {
			str_attr  = str_template;
			str_array = "template array";
			align_forbidden = TRUE;
		}
	
		type_set_for_nondata = (sp->type & TY_TYPE);
		break;
	/*}}}*/
	/*{{{  case  AT_COMMON_BLOCK :*/
	case  AT_COMMON_BLOCK :
	/*-------------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		Common block name: compatible 'attr' flags
		------------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	ok
			AT_POINTER	ok
			AT_NORMAL_VAR	ok
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		} x if external / intrinsic proc;
			AT_SUBR		} ok if internal / module /stmt proc;
					  -- handled by 'install_global_attr ()'
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	ok
			AT_NAMELIST_GROUP	ok
			AT_CONSTRUCT	ok
			AT_PROCESSORS	ok
			AT_TEMPLATE	ok
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	ok
			AT_RESULT_VAR	} x if external function, ok otherwise;
					  -- handled by 'install_global_attr ()'
			AT_IN_COMMON	ok
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		ok
			AT_OUT		ok
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
	
		/*------------------------------------------------------*/
		/* As 'AT_COMMON_BLOCK' is compatible with nearly	*/
		/* everything, it's easier to list what it's		*/
		/* *incompatible* with and negate it.			*/
		/*	If 'sp->attrs' has only compatible flags set,	*/
		/* 'install_global_entity()' is called to check that	*/
		/* 'sp' isn't already used for another global entity.	*/
		/* If it's ok, it sets the 'AT_COMMON_BLOCK' flag.	*/
		/*------------------------------------------------------*/
	
		compatible_attrs = ~(AT_PARAM | AT_MODULE | AT_PROGRAM |
		                     AT_BLOCK_DATA | AT_EXTERNAL | AT_INTRINSIC);
	
		if (only_flags_set (sp->attrs, compatible_attrs))
			return   install_global_entity (sp, AT_COMMON_BLOCK);
	
	
		attr_errmsg ( sp, AT_COMMON_BLOCK, str_common_block,
		             (sp->attrs & ~compatible_attrs));
		return  FALSE;
	
		/*------------------------------------------------------*/
		/* N.B. All incompatible flags prevent the new attr	*/
		/* being set!						*/
		/*------------------------------------------------------*/
	/*}}}*/
	/*{{{  case  AT_COMMON_SAVE :*/
	case  AT_COMMON_SAVE :
	/*------------------*/
		/*------------------------------------------------------
		SAVE /common block name/:
		-------------------------
			If the 'AT_COMMON_BLOCK' flag isn't already set,
		we try to install it, and quit if it isn't installed
		successfully (which would give an errmsg).
			Otherwise, this flag is only incompatible with
		itself (i.e. it only generates an errmsg if SAVE has
		already been specified for the common block).
		------------------------------------------------------*/
		if ( ! ((sp->attrs & AT_COMMON_BLOCK) || install_attr (sp, AT_COMMON_BLOCK)) )
			return  FALSE;
	
		else if (sp->attrs & AT_COMMON_SAVE) {
			errmsg ("SAVE has already been specified for common block %q",
			         sp->name);
			return  FALSE;
		}
		else {
			sp->attrs |= AT_COMMON_SAVE;
			return  TRUE;
		}
	/*}}}*/
	/*{{{  case  AT_IN_COMMON :*/
	case  AT_IN_COMMON :
	/*----------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		Common block variable: compatible 'attr' flags
		----------------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	ok
			AT_NORMAL_VAR	ok -- if scalar or const shape
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok  -- Seems strange, but I can't find
			AT_PUBLIC	ok  -- anything in the std to forbid it!
			AT_SAVE		--x -- Special errmsg for this!
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		
		N.B. In the cases marked '--x' above, the simple appearance
		of the name in common means that it's reinterpreted as a
		variable.  I suppose this is the best course of action.
		--------------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_POINTER | AT_NORMAL_VAR | AT_STMT_VAR |
		                    AT_COMMON_NAME | AT_EQUIV | AT_ACCESS |
		                    AT_TARGET | AT_IN_NAMELIST | AT_INFO);
	
		tolerable_attrs  = (AT_CONSTRUCT | AT_SAVE);
	
		compatible_shape = (CONST_SHAPE | DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_in_common;
		str_array = "common block array";
	
		break;
	/*}}}*/
	/*{{{  case  AT_EQUIV :*/
	case  AT_EQUIV :
	/*------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		Equivalenced object: compatible 'attr' flags
		--------------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	ok
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	ok
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_NORMAL_VAR | AT_STMT_VAR | AT_COMMON_NAME |
		                    AT_IN_COMMON | AT_EQUIV | AT_ACCESS | AT_SAVE |
		                    AT_TARGET | AT_IN_NAMELIST | AT_INFO);
		
		tolerable_attrs  = AT_CONSTRUCT;
	
		compatible_shape = (CONST_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_equiv;
		str_array = "equivalenced array";
	
		break;
	/*}}}*/
	/*{{{  case  AT_EXTERNAL :*/
	case  AT_EXTERNAL :
	/*---------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		EXTERNAL: compatible 'attr' flags
		---------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		ok
			AT_SUBR		ok
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	ok -- should be 'x', but passed so that
					   -- 'install_global_entity()'	reports
					   -- the error, as it gives a better
					   -- errmsg!
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} x  -- all external names must be
			AT_COMMON_SAVE	}    -- distinct from each other!
		
			AT_DUMMY	ok -- dummy procedure
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		--x
			AT_OUT		--x
			AT_OPTIONAL	ok -- optional dummy procedure
			AT_PRIVATE	ok -- presumably !?
			AT_PUBLIC	ok -- presumably !?
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok -- explicit interface
		--------------------------------------------------------------*/
		/*}}}*/
	
	
		if (sp->attrs & (AT_EXTERNAL | AT_INTRINSIC)) {
			/*{{{  quit with errmsg*/
			if (sp->attrs & AT_EXTERNAL)
				errmsg ("EXTERNAL attribute already specified for %q", sp->name);
			
			else
				errmsg ("incompatible attributes EXTERNAL and INTRINSIC specified for %q",
				         sp->name);
			
			return  FALSE;
			/*}}}*/
		}
	
		if ((sp->attrs & (AT_FUNC | AT_SUBR)) && (sp->attrs & AT_INFO)) {
			/*{{{  explicit interface -- check whether external/intrinsic*/
			/*--------------------------------------------------------------*/
			/* A procedure interface is already installed.  Either an	*/
			/* explicit interface was provided (which is ok), or an		*/
			/* intrinsic interface was installed as a result of a procedure	*/
			/* reference that was resolved as an intrinsic reference	*/
			/* (in which case it can't be EXTERNAL!).  The latter case is	*/
			/* distinguished by the AT_INTRINSIC flag being set in the	*/
			/* interface.							*/
			/*								*/
			/* N.B. The latter case *doesn't* mean that 'sp->attrs' has	*/
			/* AT_INTRINSIC set -- that's only done as a result of an	*/
			/* 'INTRINSIC' declaration.					*/
			/*--------------------------------------------------------------*/
			if (sp->info.sp->attrs & AT_INTRINSIC) {
				errmsg ("%q was established to be an intrinsic procedure in a previous reference, so cannot be EXTERNAL",
				         sp->name);
				return  FALSE;
			}
			else {
				sp->attrs |= AT_EXTERNAL;	/* we know it's ok, and is already
								   installed in 'global_list' if it
								   isn't a dummy procedure!  */
				return  TRUE;
			}
			/*}}}*/
		}
	
		if (sp == this_unitp->sp) {
			/*{{{  host unit declared EXTERNAL; give errmsg & return FALSE*/
			/*--------------------------------------------------------------*/
			/* N.B. This errmsg is given even if the host is a module or	*/
			/* main program, for which EXTERNAL is invalid anyway.  But I	*/
			/* guess it's reasonably clear!					*/
			/*--------------------------------------------------------------*/
			errmsg ("cannot specify EXTERNAL for the host %s %q",
			         ((sp->attrs & (AT_SUBR | AT_FUNC | AT_RESULT_VAR)) ?
			                                       "procedure" : "program unit"),
			         sp->name);
			return  FALSE;
			/*}}}*/
		}
		
		/*------------------------------------------------------*/
		/* no interface (though may have been referenced)...	*/
		/*------------------------------------------------------*/
		compatible_attrs = (AT_STMT_VAR | AT_FUNC | AT_SUBR | AT_BLOCK_DATA |
		                    AT_DUMMY | AT_OPTIONAL | AT_ACCESS | AT_INFO);
	
	try_to_set_external :
	
		if (only_flags_set (sp->attrs, compatible_attrs)) {
			/*{{{  install in 'global_list' if non-dummy, & set flags if OK*/
			/*----------------------------------------------------------------
			AT_EXTERNAL declares 'sp' to be an external procedure or block
			data unit.
				If it isn't a dummy proc, then it must have a different
			name from all other global entities, so 'install_global_entity()'
			checks that it doesn't clash with global entities in other scope
			units.  (E.g.:  is 'sp->name' already used for another global
			entity?  If it's known locally to be a function or subroutine,
			is it used likewise in other scope units?)  If these tests are
			passed, 'install_global_entity()' installs 'sp' in the
			'global_list', sets AT_EXTERNAL in 'sp->attrs', and returns TRUE.
			Otherwise it does nothing (e.g. it doesn't set AT_EXTERNAL)
			and returns FALSE.
				In the former case, the function may also have the
			side-effect of installing the 'proc type', AT_FUNC or AT_SUBR,
			in 'sp->attrs' as well, if it isn't set but can be determined
			from its previous use in other scope units.
				If a shape is specified, report an error (provided that
			AT_FUNC isn't set, in which case the error would have been reported
			already) but still set the flags as before.
			----------------------------------------------------------------*/
			if (sp->arp  &&  ! (sp->attrs & AT_FUNC))
				/*------------------------------------------------------*/
				/* Give this errmsg even if it's subsequently determined*/
				/* (by 'install_global_entity()') that it's already an	*/
				/* external *subroutine*.  (Do this to clarify *why*	*/
				/* any subsequent errmsg identifies it as a function.)	*/
				/*------------------------------------------------------*/
				errmsg ("array dimensions for external function %q can only be specified in an explicit interface",
				         sp->name);
			
			/*{{{  if 'sp->alignp / distp' set, give errmsg & delete it*/
			if (sp->alignp) {
				earlier_error = TRUE;
				if (sp->alignp->tgt_sp)
					errmsg ("external procedure %q cannot be aligned", sp->name);
				else
					errmsg ("cannot specify INHERIT for external procedure %q",
					         sp->name);
				delete_align (sp->alignp, DECR_NLINKS);
				sp->alignp = NULL_Align;
				earlier_error = FALSE;
			}
			
			if (sp->distp) {
				earlier_error = TRUE;
				errmsg ("external procedure %q cannot be distributed", sp->name);
				delete_distr (sp->distp, DECR_NLINKS);
				sp->distp = NULL_Distr;
				earlier_error = FALSE;
			}
			/*}}}*/
			
			if ( ! first_try)
				clear_flags (sp->attrs, AT_CONSTRUCT);
			
			if (sp->attrs & AT_DUMMY) {
				sp->attrs |= AT_EXTERNAL;
			
				if ((sp->type & TY_TYPE)  ||  sp->arp)
					sp->attrs |= AT_FUNC;
			
				return  TRUE;
			}
			else
				return  install_global_entity (sp, AT_EXTERNAL);
			/*}}}*/
		}
	
		/*---------------------*/
		/* Failed!  Try again? */
		/*---------------------*/
		if (first_try) {
			attr_errmsg (sp, AT_EXTERNAL, str_external,
			                         (sp->attrs & ~compatible_attrs));
			/*------------------------------------------------------*/
			/* the following incompatible flags don't prevent the	*/
			/* new attr being set...				*/
			/*------------------------------------------------------*/
			compatible_attrs |= (AT_CONSTRUCT | AT_INTENT | AT_SAVE |
			                     AT_TARGET);
			first_try = FALSE;
			goto  try_to_set_external;
		}
		else
			return  FALSE;
	/*}}}*/
	/*{{{  case  AT_INTRINSIC :*/
	case  AT_INTRINSIC :
	/*----------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		INTRINSIC: compatible 'attr' flags
		----------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	x
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		ok
			AT_SUBR		ok
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} x  -- common block and intrinsic
			AT_COMMON_SAVE	}    -- names must be distinct
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	x
			AT_EQUIV	x
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		--x
			AT_TARGET	--x
		
			AT_IN_NAMELIST	x
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
		
		if (sp->attrs & (AT_EXTERNAL | AT_INTRINSIC)) {
			/*{{{  give errmsg and return FALSE*/
			
			if (sp->attrs & AT_INTRINSIC)
				errmsg ("INTRINSIC attribute already specified for %q", sp->name);
			
			else
				errmsg ("incompatible attributes INTRINSIC and EXTERNAL specified for %q",
				         sp->name);
			
			return  FALSE;
			/*}}}*/
		}
		else if (sp->attrs & (AT_FUNC | AT_SUBR)) {
			if (sp->attrs & AT_INFO) {
				/*{{{  explicit interface.  Return TRUE if intrinsic, else FALSE*/
				/*--------------------------------------------------------------*/
				/* A procedure interface is already installed.  Either an	*/
				/* explicit interface was provided (an error!), or an intrinsic	*/
				/* interface was installed as a result of a procedure reference	*/
				/* that was resolved as an intrinsic reference (which is ok!).	*/
				/* The latter case is distinguished by the AT_INTRINSIC flag	*/
				/* being set in the interface.					*/
				/*								*/
				/* N.B. The latter case *doesn't* mean that 'sp->attrs' has	*/
				/* AT_INTRINSIC set -- that's only done as a result of an	*/
				/* 'INTRINSIC' declaration.					*/
				/*--------------------------------------------------------------*/
				if (sp->info.sp->attrs & AT_INTRINSIC) {
					sp->attrs |= AT_INTRINSIC;	/* we know it's ok! */
					return  TRUE;
				}
				else {
					errmsg ("must not specify an interface body for an intrinsic procedure: %q",
					         sp->name);
					return  FALSE;
				}
				/*}}}*/
			}
			else if (sp == this_unitp->sp) {
				/*{{{  host proc declared INTRINSIC; give errmsg & return FALSE*/
				errmsg ("cannot specify INTRINSIC for the host procedure %q", sp->name);
				return  FALSE;
				/*}}}*/
			}
			else {
				/*{{{  referenced & found to be non-intrinsic.  Return FALSE*/
				/*--------------------------------------------------------------*/
				/* One of the flags AT_FUNC or AT_SUBR is set, but not AT_INFO.	*/
				/*	This means that the procedure has been referenced and	*/
				/* found *not* to be intrinsic (or at least, not an intrinsic	*/
				/* of the appropriate type -- function or subroutine -- for the	*/
				/* reference).  Thus it can't be intrinsic.			*/
				/*--------------------------------------------------------------*/
				if (sp->attrs & AT_DUMMY)
					errmsg ("dummy argument %q cannot be an intrinsic procedure", sp->name);
				else
					errmsg ("%q was established to be an external, not intrinsic, procedure in a previous reference",
					         sp->name);
				return  FALSE;
				/*}}}*/
			}
		}
		/*------------------------------------------------------*/
		/* ...else no previous reference or explicit interface	*/
		/*------------------------------------------------------*/
	
		compatible_attrs = (AT_STMT_VAR | AT_FUNC | AT_SUBR | AT_ACCESS |
		                    AT_INFO);
	
	try_to_set_intrinsic :
	
		if (only_flags_set (sp->attrs, compatible_attrs)) {
			Symbol  *intrin_sp;
	
			if (intrin_sp = lookup_intrin_proc (sp->name)) {
				/*{{{  found intrinsic; install AT_INTRINSIC, AT_INFO, AT_FUNC/SUBR*/
				/*--------------------------------------------------------------*/
				/* It's an intrinsic proc.  Install (AT_INTRINSIC | AT_INFO)	*/
				/* and AT_FUNC or AT_SUBR, and if it has been typed or		*/
				/* dimensioned, check for consistency.				*/
				/*--------------------------------------------------------------*/
				sp->info.sp = intrin_sp;
				sp->attrs  |= ( AT_INTRINSIC | AT_INFO |
				               ((intrin_sp->attrs & AT_RESULT_VAR) ? AT_FUNC : AT_SUBR) );
				
				if (sp->attrs & AT_SUBR) {
					/*-------------------------*/
					/* an intrinsic subroutine */
					/*-------------------------*/
					if ((sp->type & TY_TYPE) || sp->arp) {
						begin_errmsg ("intrinsic subroutine %q has been given ", sp->name);
						if (sp->type & TY_TYPE) {
							end_errmsg (sp->arp ? "a data type and array dimensions" :
							                      "a data type");
							clear_flags (sp->type, TY_TYPE);
						}
						else
							end_errmsg ("array dimensions");
						if (sp->arp) {
							delete_array (sp->arp, DECR_NLINKS);
							sp->arp = NULL_Array;
						}
					}
				}
				else {
					/*-----------------------*/
					/* an intrinsic function */
					/*-----------------------*/
					if (  (sp->type & TY_TYPE)  &&
					    ! (sp->type & intrin_fn_result_types (intrin_sp)) ) {
						errmsg ("intrinsic function %q incorrectly typed", sp->name);
						clear_flags (sp->type, TY_TYPE);
					}
					if (sp->arp)
						errmsg ("array dimensions must not be specified for an intrinsic function: %q",
						         sp->name);
				}
				
				
				/*{{{  if 'sp->alignp / distp' set, give errmsg & delete it*/
				if (sp->alignp) {
					earlier_error = TRUE;
					if (sp->alignp->tgt_sp)
						errmsg ("intrinsic %s %q cannot be aligned",
						         ((sp->attrs & AT_FUNC) ? "function" : "subroutine"),
						          sp->name);
					else
						errmsg ("cannot specify INHERIT for intrinsic %s %q",
						         ((sp->attrs & AT_FUNC) ? "function" : "subroutine"),
						          sp->name);
					delete_align (sp->alignp, DECR_NLINKS);
					sp->alignp = NULL_Align;
					earlier_error = FALSE;
				}
				
				if (sp->distp) {
					earlier_error = TRUE;
					errmsg ("intrinsic %s %q cannot be distributed",
					         ((sp->attrs & AT_FUNC) ? "function" : "subroutine"),
					          sp->name);
					delete_distr (sp->distp, DECR_NLINKS);
					sp->distp = NULL_Distr;
					earlier_error = FALSE;
				}
				/*}}}*/
				
				if (! first_try)
					clear_flags (sp->attrs, AT_CONSTRUCT);
				
				return  TRUE;
				/*}}}*/
			}
			else {
				errmsg ("unknown intrinsic procedure: %q", sp->name);
				return  FALSE;
			}
		}
		/*---------------------*/
		/* Failed!  Try again? */
		/*---------------------*/
		if (first_try) {
			attr_errmsg ( sp, AT_INTRINSIC, str_intrinsic,
			                         (sp->attrs & ~compatible_attrs));
			/*------------------------------------------------------*/
			/* the following incompatible flags don't prevent the	*/
			/* new attr being set...				*/
			/*------------------------------------------------------*/
			compatible_attrs |= (AT_CONSTRUCT | AT_SAVE | AT_TARGET);
			first_try = FALSE;
			goto  try_to_set_intrinsic;
		}
		else
			return  FALSE;
	/*}}}*/
	/*{{{  case  AT_IN :  case  AT_OUT :  case  AT_INOUT :*/
	case  AT_IN :
	case  AT_OUT :
	case  AT_INOUT :
	/*------------*/
		/*------------------------------------------------------*/
		/* N.B. Effectively any shape is allowed for a dummy	*/
		/* argument.						*/
		/*------------------------------------------------------*/
		if (sp->attrs & AT_DUMMY) {
			if ( ! (sp->attrs & (AT_PROCEDURE | AT_POINTER | AT_INTENT)) ) {
				sp->attrs |= attr;
				return  TRUE;
			}
			else if (sp->attrs & AT_PROCEDURE)
				errmsg ("cannot specify INTENT for a dummy procedure: %q", sp->name);
	
			else if (sp->attrs & AT_POINTER)
				errmsg ("cannot specify INTENT for a dummy pointer: %q", sp->name);
	
			else
				errmsg ("INTENT already specified for %q", sp->name);
		}
		else
			errmsg ("INTENT specified for non dummy argument %q", sp->name);
	
		return  FALSE;
	/*}}}*/
	/*{{{  case  AT_OPTIONAL :*/
	case  AT_OPTIONAL :
	/*---------------*/
		/*------------------------------------------------------*/
		/* N.B. Effectively any shape is allowed for a dummy	*/
		/* argument.						*/
		/*------------------------------------------------------*/
		if (sp->attrs & AT_DUMMY) {
			if ( !(sp->attrs & AT_OPTIONAL) ) {
				sp->attrs |= AT_OPTIONAL;
				return  TRUE;
			}
			else
				errmsg ("OPTIONAL attribute already specified for %q", sp->name);
		}
		else
			errmsg ("OPTIONAL attribute specified for non dummy argument %q", sp->name);
	
		return  FALSE;
	/*}}}*/
	/*{{{  case  AT_PRIVATE :  case  PUBLIC : -- don't set if incompat*/
	case  AT_PRIVATE :
	case  AT_PUBLIC :
	/*--------------*/
		/*{{{  COMMENT: compatible attrs -- sort of ok!*/
		/*--------------------------------------------------------------
		PRIVATE or PUBLIC: compatible 'attr' flags
		------------------------------------------
		N.B. We don't bother setting this flag if it's inappropriate!
		
		ok  = compatible;
		x   = incompatible, and don't set new flag.
		--x = incompatible, but still set flag. (Doesn't apply here.)
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	ok
			AT_POINTER	ok
			AT_NORMAL_VAR	ok
			AT_STMT_VAR	ok
		
			AT_PARAM	ok
		
			AT_FUNC		ok
			AT_SUBR		ok
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	ok
			AT_NAMELIST_GROUP	ok
			AT_CONSTRUCT	x
			AT_PROCESSORS	ok -- *** really should be illegal as these     ***
			AT_TEMPLATE	ok -- *** should only appear in HPF directives! ***
		
			AT_COMMON_BLOCK	} ok -- though it doesn't apply to this!
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x } -- but doesn't arise, since won't
			AT_RESULT_VAR	x } -- be in a module.
			AT_IN_COMMON	ok -- but I'm not sure about this!
			AT_EQUIV	ok
		
			AT_EXTERNAL	ok -- but I'm not sure about this!
			AT_INTRINSIC	ok -- but I'm not sure about this!
			AT_IN		x -- see 'AT_DUMMY'
			AT_OUT		x --  "       "
			AT_OPTIONAL	x --  "       "
			AT_PRIVATE	x
			AT_PUBLIC	x
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		
		/*}}}*/
	
		/*------------------------------------------------------*/
		/* N.B.  'yyparse()' checks that we're in a module	*/
		/* before calling this.					*/
		/*------------------------------------------------------*/
	
		compatible_attrs = (AT_ALLOC | AT_POINTER | AT_NORMAL_VAR | AT_STMT_VAR |
		                    AT_PARAM | AT_FUNC | AT_SUBR | AT_DERIVED_TYPE |
		                    AT_NAMELIST_GROUP | AT_PROCESSORS | AT_TEMPLATE |
		                    AT_COMMON_NAME | AT_IN_COMMON | AT_EQUIV |
		                    AT_EXTERNAL | AT_INTRINSIC | AT_SAVE | AT_TARGET |
		                    AT_IN_NAMELIST | AT_INFO);
	
		tolerable_attrs  = 0;
		
		compatible_shape = SHAPE_MASK;		/* i.e. any shape OK */
	
		str_attr = ((attr == AT_PRIVATE) ? str_private_attr : str_public_attr);
	
		break;
	/*}}}*/
	/*{{{  case  AT_SAVE :*/
	case  AT_SAVE :
	/*-----------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		SAVE: compatible 'attr' flags
		-----------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	ok
			AT_POINTER	ok
			AT_NORMAL_VAR	ok -- if scalar or const shape
			AT_STMT_VAR	ok
		
			AT_PARAM	ok -- strange, but doesn't appear to be
					      disallowed by any rule in F90!
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok -- though it doesn't apply to this!
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	x -- special errmsg for this!
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		x
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_ALLOC | AT_POINTER | AT_NORMAL_VAR |
		                    AT_STMT_VAR | AT_PARAM | AT_COMMON_NAME |
		                    AT_EQUIV | AT_ACCESS | AT_TARGET |
		                    AT_IN_NAMELIST | AT_INFO);
	
		tolerable_attrs  = AT_CONSTRUCT;
	
		compatible_shape = (CONST_SHAPE | DEFERRED_SHAPE | INVALID_SHAPE);
	
		str_attr  = str_save_attr;
		str_array = "SAVEd array";
	
		break;
	/*}}}*/
	/*{{{  case  AT_TARGET :*/
	case  AT_TARGET :
	/*-------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		TARGET: compatible 'attr' flags
		-------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	ok
			AT_POINTER	x
			AT_NORMAL_VAR	ok
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok -- though it doesn't apply to this!
			AT_COMMON_SAVE	}
		
			AT_DUMMY	ok
			AT_RESULT_VAR	ok
			AT_IN_COMMON	ok
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		ok
			AT_OUT		ok
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	x
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
		
		compatible_attrs = (AT_ALLOC | AT_NORMAL_VAR | AT_STMT_VAR |
		                    AT_COMMON_NAME | AT_DUMMY | AT_RESULT_VAR |
		                    AT_IN_COMMON | AT_EQUIV | AT_INTENT | AT_OPTIONAL |
		                    AT_ACCESS | AT_SAVE | AT_IN_NAMELIST | AT_INFO);
		
		tolerable_attrs  = AT_CONSTRUCT;
	
		compatible_shape = SHAPE_MASK;		/* i.e. any shape is OK */
	
		str_attr = str_target_attr;
	
		break;
	/*}}}*/
	/*{{{  case  AT_IN_NAMELIST : -- sets type if not yet declared*/
	case  AT_IN_NAMELIST :
	/*------------------*/
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		Object in a namelist group:  compatible 'attr' flags
		----------------------------------------------------
		N.B. We don't bother setting this flag if it's inappropriate!
		
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	ok -- if scalar or const shape
			AT_STMT_VAR	ok
		
			AT_PARAM	x
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	x
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	ok
			AT_RESULT_VAR	ok
			AT_IN_COMMON	ok
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		ok
			AT_OUT		ok
			AT_OPTIONAL	ok
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok
		--------------------------------------------------------------*/
		/*}}}*/
	
		/*------------------------------------------------------*/
		/* N.B. A side-effect of installing this flag is that	*/
		/* an implicit data-type is installed if the data-type	*/
		/* isn't already declared.				*/
		/*------------------------------------------------------*/
		
		compatible_attrs = (AT_NORMAL_VAR | AT_STMT_VAR | AT_COMMON_NAME |
		                    AT_DUMMY | AT_RESULT_VAR | AT_IN_COMMON |
		                    AT_EQUIV | AT_INTENT | AT_OPTIONAL | AT_ACCESS |
		                    AT_SAVE | AT_TARGET | AT_IN_NAMELIST | AT_INFO);
		
		compatible_shape = (CONST_SHAPE | INVALID_SHAPE);
		
		if (only_flags_set (sp->attrs, compatible_attrs)) {
			/*------------------------------------------------------*/
			/* No incompatible 'attr' flags.  If there's a shape	*/
			/* error, report it but still set the 'attr' flag.	*/
			/*------------------------------------------------------*/
			if (sp->arp  &&  ! (sp->arp->attrs & compatible_shape))
				errmsg ("only constant shape arrays can be in a namelist group: %q",
				         sp->name);
	
			sp->attrs |= AT_IN_NAMELIST;
			lookup_or_install_type (sp);
			return  TRUE;
		}
	
		attr_errmsg (sp, AT_IN_NAMELIST, str_in_namelist,
		                                   (sp->attrs & ~compatible_attrs));
		return  FALSE;
	/*}}}*/
	/*{{{  case  AT_INIT : (i.e. AT_INFO) -- sets type if not yet declared*/
	case  AT_INIT :
	/*-----------*/
		/*--------------------------------------------------------
			'AT_INIT' is one of the aliases for 'AT_INFO';
		when it's installed here, it always stands for this alias.
		Furthermore, this function is only invoked prior to initialising
		a *variable*;  for a PARAMETER, 'yyparse()' itself performs
		all of the necessary checks and installs the 'AT_INIT' flag.
			Side-effects of installing this flag are:
		--  'AT_NORMAL_VAR' is also installed (since it's a normal
		variable, not a parameter -- see above);
		--  if the data-type isn't already declared, an implicit
		data-type is installed.
		--------------------------------------------------------*/
		
		/*{{{  COMMENT: compatible attrs -- ok*/
		/*--------------------------------------------------------------
		Initialised variable:  compatible 'attr' flags
		----------------------------------------------
		ok  = compatible;
		x   = incompatible, and don't set new flag;
		--x = incompatible, but still set new flag.
		o   = incompatible, but replace old flag by new.
		
			AT_ALLOC	x
			AT_POINTER	x
			AT_NORMAL_VAR	ok -- if scalar or const shape.
					   -- N.B. this means AT_INIT also set!  However,
					   -- can be initialised in multiple stages in a
					   -- DATA stmt.
			AT_STMT_VAR	ok
		
			AT_PARAM	x -- but *cannot* be set -- see intro.
		
			AT_FUNC		x
			AT_SUBR		x
			AT_MODULE	x
			AT_PROGRAM	x
			AT_BLOCK_DATA	x
		
			AT_DERIVED_TYPE	x
			AT_NAMELIST_GROUP	x
			AT_CONSTRUCT	o
			AT_PROCESSORS	x
			AT_TEMPLATE	x
		
			AT_COMMON_BLOCK	} ok
			AT_COMMON_SAVE	}
		
			AT_DUMMY	x
			AT_RESULT_VAR	x
			AT_IN_COMMON	ok -- but only in BLOCK DATA
			AT_EQUIV	ok
		
			AT_EXTERNAL	x
			AT_INTRINSIC	x
			AT_IN		x
			AT_OUT		x
			AT_OPTIONAL	x
			AT_PRIVATE	ok
			AT_PUBLIC	ok
			AT_SAVE		ok
			AT_TARGET	ok
		
			AT_IN_NAMELIST	ok
			AT_INFO		ok -- see AT_NORMAL_VAR
		--------------------------------------------------------------*/
		/*}}}*/
	
		compatible_attrs = (AT_NORMAL_VAR | AT_STMT_VAR | AT_COMMON_NAME |
		                    AT_IN_COMMON | AT_EQUIV | AT_ACCESS | AT_SAVE |
		                    AT_TARGET | AT_IN_NAMELIST | AT_INFO);
	
		compatible_shape = (CONST_SHAPE | INVALID_SHAPE);
	
		str_array = "can only initialise constant shape arrays: %q";
		                     /* ...N.B. wrong form for 'shape_errmsg ()' */
	try_to_set_init :
	
		if (only_flags_set (sp->attrs, compatible_attrs)) {
			if (sp->arp  &&  ! (sp->arp->attrs & compatible_shape))
				errmsg (str_array,  sp->name);
	
			sp->attrs |= (AT_INIT | AT_NORMAL_VAR);
			lookup_or_install_type (sp);
			if (! first_try)
				clear_flags (sp->attrs, AT_CONSTRUCT);
			return  TRUE;
		}
	
		/*---------------------*/
		/* Failed!  Try again? */
		/*---------------------*/
		if (first_try) {
			attr_errmsg (sp, AT_INIT, str_init,
			                             (sp->attrs & ~compatible_attrs));
			/*------------------------------------------------------*/
			/* the following incompatible flags don't prevent the	*/
			/* new attr being set...				*/
			/*------------------------------------------------------*/
			compatible_attrs |= AT_CONSTRUCT;
			first_try = FALSE;
			goto  try_to_set_init;
		}
		else
			return  FALSE;
	/*}}}*/
	}
	/*{{{  do checking!*/
	try_to_set_attr :
	
	if (only_flags_set (sp->attrs, compatible_attrs)  &&  ! type_set_for_nondata) {
		/*------------------------------------------------------*/
		/* The new 'attr' flag is compatible with existing	*/
		/* attrs, and with the existence of a datatype and/or	*/
		/* shape if present.  If there's a shape error, report	*/
		/* it but still set the 'attr' flag.			*/
		/*------------------------------------------------------*/
		if (sp->arp  &&  ! (sp->arp->attrs & compatible_shape))
			shape_errmsg (sp, str_array, compatible_shape);
	
		/*{{{  if 'sp->alignp / distp' set but disallowed, give errmsg & delete it*/
		if (sp->alignp  &&  align_forbidden) {
			earlier_error = TRUE;
			if (sp->alignp->tgt_sp)
				errmsg ("cannot ALIGN %s: %q", str_attr, sp->name);
			else
				errmsg ("cannot specify INHERIT for %s: %q", str_attr, sp->name);
			delete_align (sp->alignp, DECR_NLINKS);
			sp->alignp = NULL_Align;
			earlier_error = FALSE;
		}
		
		if (sp->distp  &&  distr_forbidden) {
			earlier_error = TRUE;
			errmsg ("cannot DISTRIBUTE %s: %q", str_attr, sp->name);
			delete_distr (sp->distp, DECR_NLINKS);
			sp->distp = NULL_Distr;
			earlier_error = FALSE;
		}
		/*}}}*/
	
		sp->attrs |= attr;
		if (set_type)
			lookup_or_install_type (sp);
	
		if (! first_try)
			clear_flags (sp->attrs, AT_CONSTRUCT);
		return  TRUE;
	}
	
	/*---------------------*/
	/* Failed!  Try again? */
	/*---------------------*/
	if (first_try) {
		attr_errmsg (sp, attr, str_attr, (sp->attrs & ~compatible_attrs));
	
		if (type_set_for_nondata)
			return  FALSE;
	
		/*------------------------------------------------------*/
		/* the presence of 'tolerable_attrs' flags in		*/
		/* 'sp->attrs' doesn't prevent the new attr being set...*/
		/*------------------------------------------------------*/
		compatible_attrs |= tolerable_attrs;
		first_try = FALSE;
		goto  try_to_set_attr;
	}
	else
		return  FALSE;
	/*}}}*/
}
/*}}}*/

/*{{{  attr_errmsg (sp, new_attr, new_str, incompatible_attrs) -- OK*/
attr_errmsg (sp, new_attr, new_str, incompatible_attrs)
/*---------------------------------------------------*/
	Symbol  *sp;
	int32    new_attr;
	char    *new_str;
	int32    incompatible_attrs;	/* if 0, error is the presence
					   of a data type or shape */
{
	char  *name = sp->name;
	char  *old_str;
	int32  either_attr,  old_attr;

/*{{{  #define  AT_OBJECT, AT_OTHER_ATTRS, AT_IN_PLACE*/
/*--------------------------------------------------------------*/
/* N.B. The following groupings omit:				*/
/*								*/
/*	AT_COMMON_BLOCK,  AT_DUMMY,  AT_EQUIV,  AT_INFO		*/
/*								*/
/* (and AT_COMMON_SAVE, which is never set in either 'new_attr'	*/
/* or 'incompatible_attrs' anyway!)				*/
/*--------------------------------------------------------------*/
#define  AT_OBJECT		(AT_ENTITY | AT_EXTERNAL | AT_INTRINSIC)
#define  AT_IN_PLACE		(AT_IN_COMMON | AT_IN_NAMELIST)
#define  AT_OTHER_ATTRS		(AT_ATTRS  &  ~(AT_EXTERNAL | AT_INTRINSIC))

/*}}}*/

/*************************************************************
-- ERRMSGS ALWAYS REFER TO: "an initialised variable".
WOULD IT BE BETTER TO SAY, E.G. "a static variable" ?
************************************************************/
/*{{{  Notes*/
/**
Notes
-----
(i)  new_attr, old_attr != AT_OPTIONAL, AT_COMMON_SAVE

because:
-- this function isn't invoked with these values of 'new_attr';
-- If OPTIONAL is set in 'incompatible_attrs' then so is AT_DUMMY,
and if COMMON_SAVE is set then so is AT_COMMON_BLOCK, and 'old_attr'
is set to the latter in preference to the former.

(ii)  new-attr != AT_PROGRAM, AT_BLOCK_DATA, AT_DUMMY, AT_INTENT

(iii) old_attr != AT_INFO (i.e. AT_INIT).

as it's never set in 'incompatible_attrs'.

 --**/
/*}}}*/

	if (! incompatible_attrs) {
		/*{{{  error must relate to 'sp' having a datatype or shape*/
		/*--------------------------------------------------------------*/
		/* There are no incompatible attributes, so the error must	*/
		/* relate to 'name' having a declared datatype or shape		*/
		/* (i.e. 'type' or 'arp' is defined).				*/
		/*--------------------------------------------------------------*/
		if (sp->arp)
			errmsg ("%q is an array entity, so cannot be %s", name, new_str);
		
		else if (sp->type & TY_TYPE)
			errmsg ("%q has a data type, so cannot be %s", name, new_str);
		
		return;
		/*}}}*/
	}
	/*{{{  isolate 'old_attr' from 'incompatible_attrs', & define 'old_str'*/
	/*----------------------------------------------------------------
	Set 'old_attr' to the 'highest priority' flag set in
	'incompatible_attrs', and define 'old_str' accordingly.  By the
	'highest priority' flag, we mean that we try to select the one
	that is the origin of the incompatibility, rather than one that is
	consequent on it, e.g. an AT_ENTITY flag is selected in preference
	to an AT_ATTR one.
	 	We don't check for the presence of:
	
	    AT_INFO, AT_COMMON_SAVE, AT_OPTIONAL
	
	because:
	-- AT_INFO is never set in 'incompatible_attrs';
	
	-- although AT_COMMON_SAVE and AT_OPTIONAL may be set in
	'incompatible_attrs', they do not cause incompatibility
	themselves;  rather the blame would lie with AT_COMMON_BLOCK
	or AT_DUMMY resp, which always accompany them.
	----------------------------------------------------------------*/
	
	if (incompatible_attrs & AT_ENTITY) {	/* N.B. *not* AT_OBJECT */
		/*{{{  various possibilities!*/
		/*--------------------------------------------------------------*/
		/* N.B.  AT_ENTITY excludes the attribute flags AT_EXTERNAL and	*/
		/* AT_INTRINSIC (which are included by AT_OBJECT).		*/
		/* It's guaranteed that only one *entity* flag will be set.	*/
		/* On the other hand, one of AT_EXTERNAL or AT_INTRINSIC may be	*/
		/* set together with the entity flags AT_FUNC or AT_SUBR, in	*/
		/* which case the latter take precedence.			*/
		/*--------------------------------------------------------------*/
		
		switch (old_attr = (incompatible_attrs & AT_ENTITY)) {
		case AT_ALLOC :		old_str = str_alloc;		break;
		case AT_POINTER :	old_str = str_pointer;		break;
		case AT_NORMAL_VAR :	old_str = str_normal_var;	break;
		case AT_PARAM :		old_str = str_param;		break;
		case AT_FUNC :		old_str = str_func;		break;
		case AT_SUBR :		old_str = str_subr;		break;
		case AT_MODULE :	old_str = str_module;		break;
		case AT_PROGRAM :	old_str = str_program;		break;
		case AT_BLOCK_DATA :	old_str = str_block_data;	break;
		case AT_DERIVED_TYPE :	old_str = str_derived_type;	break;
		case AT_NAMELIST_GROUP:	old_str = str_namelist_group;	break;
		case AT_CONSTRUCT :	old_str = str_construct;	break;
		case AT_PROCESSORS :	old_str = str_processors;	break;
		case AT_TEMPLATE :	old_str = str_template;		break;
		}
		/*}}}*/
	}
	else if (incompatible_attrs & AT_EXTERNAL) {
		old_attr = AT_EXTERNAL;  old_str = str_external;
	}
	else if (incompatible_attrs & AT_INTRINSIC) {
		old_attr = AT_INTRINSIC;  old_str = str_intrinsic;
	}
	else {
		old_attr = AT_COMMON_BLOCK;
		while ( !(incompatible_attrs & old_attr))
		      old_attr <<= 1;
	
		switch (old_attr) {
		/*------------------------------------------------------------*/
		/* can't be AT_COMMON_SAVE, AT_OPTIONAL, AT_INFO -- see above */
		/*------------------------------------------------------------*/
		case  AT_COMMON_BLOCK :	old_str = str_common_block;	break;
		case  AT_DUMMY :	old_str = str_dummy;		break;
		case  AT_RESULT_VAR :	old_str = str_result_var;	break;
		case  AT_IN_COMMON :	old_str = str_in_common;	break;
		case  AT_EQUIV :	old_str = str_equiv;		break;
		case  AT_IN :		old_str = str_intent_attr;	break;
		case  AT_OUT :		old_str = str_intent_attr;	break;
		case  AT_PRIVATE :	old_str = str_private_attr;	break;
		case  AT_PUBLIC :	old_str = str_public_attr;	break;
		case  AT_SAVE :		old_str = str_save_attr;	break;
		case  AT_TARGET :	old_str = str_target_attr;	break;
		case  AT_IN_NAMELIST :	old_str = str_in_namelist;	break;
		}
	}
	
	/*}}}*/

	either_attr = old_attr | new_attr;

	if (new_attr & old_attr) {
		/*{{{  duplicated declaration*/
		/*--------------------------------------------------------------*/
		/* Duplicated 'attr' flag.  It must be one of:			*/
		/*						AT_EXTERNAL	*/
		/*						AT_INTRINSIC	*/
		/*	AT_ALLOC	AT_CONSTRUCT		AT_PRIVATE	*/
		/*	AT_POINTER	AT_PROCESSORS		AT_PUBLIC	*/
		/*	AT_PARAM	AT_TEMPLATE		AT_SAVE		*/
		/*	AT_DERIVED_TYPE				AT_TARGET	*/
		/*	|____________________________|		|__________|	*/
		/*               (AT_ENTITY)			 (AT_ATTRS)	*/
		/* or								*/
		/*	AT_IN_COMMON						*/
		/*--------------------------------------------------------------*/
		
		switch (new_attr) {
		case  AT_DERIVED_TYPE :
			errmsg ("repeated definition of derived type %q", name);
			return;
			
		case  AT_CONSTRUCT :
			errmsg ("same construct name %q is used for different constructs", name);
			return;
		
		case  AT_IN_COMMON :
			errmsg ("multiple appearance of %q in common", name);
			return;
		}
		
		if (new_attr & AT_ENTITY)
			/*-------------------------------------------*/
			/* N.B. this *excludes* EXTERNAL & INTRINSIC */
			/*-------------------------------------------*/
			errmsg ("%q already declared as %s", name, new_str);
		
		else if (new_attr & AT_ATTRS) {
			/*-------------------------------------------*/
			/* N.B. this *includes* EXTERNAL & INTRINSIC */
			/*-------------------------------------------*/
			if (new_attr == AT_EXTERNAL)
				new_str = str_external_attr;	/* not 'str_external' */
			else if (new_attr == AT_INTRINSIC)
				new_str = str_intrinsic_attr;	/* not 'str_intrinsic' */
		
			errmsg ("%s attribute already specified for %q", new_str, name);
		}
		/*}}}*/
	}		
	else if ( (new_attr & (AT_OBJECT | AT_COMMON_BLOCK)) &&
	          (old_attr & (AT_OBJECT | AT_COMMON_BLOCK)) )
		errmsg ("%q is %s; cannot use the same name for %s",
	                                            name, old_str, new_str);

	else if ((new_attr & AT_OTHER_ATTRS) && (old_attr & AT_OTHER_ATTRS))
		errmsg ("incompatible attributes %s and %s specified for %q",
		                                    new_str, old_str, name);
	else if (either_attr & AT_IN_PLACE) {
/******************************************************************
ERRMSG IMPLIES THAT THE OBJECT HAS THE STATED ENTITY TYPE, WHICH
IS MISLEADING IF IT ISN'T SET DUE TO ERROR.  (E.G. IF WE TRY TO SET
'AT_FUNC' WHEN 'AT_IN_COMMON' IS SET, THE FORMER *ISN'T* SET!)
*******************************************************************/
		if (either_attr & (AT_OBJECT | AT_DUMMY | AT_RESULT_VAR)) {
			/*{{{  e.g. "allocatable array 'A' cannot be in a common block"*/
			char  *place_str  = (new_attr & AT_IN_PLACE) ? new_str : old_str;
			char  *object_str = (new_attr & AT_IN_PLACE) ? old_str : new_str;
			
			/*--------------------------------------------------------------*/
			/* Skip over the leading article ("a ", "an " or "the ") in the	*/
			/* 'object string'.						*/
			/* N.B. This assumes that *all* such strings begin with "a",	*/
			/* "an" or "the"!						*/
			/*--------------------------------------------------------------*/
			while (*object_str++ != ' ')
				; 
			
			errmsg ("%s %q cannot be %s", object_str, name, place_str);
			/*}}}*/
		}
		else if (either_attr & AT_SAVE)
			/*{{{  "cannot SAVE an individual common block variable: 'X'"*/
			errmsg ("cannot SAVE an individual common block variable: %q", name);
			/*}}}*/
	}
	else if (either_attr & (AT_EQUIV | AT_INIT)) {
		/*{{{  e.g. "cannot equivalence a pointer: 'P'"*/
		/*------------------------------------------------------*/
		/* 'object_str' is 'a dummy argument' or an entity type.*/
		/*------------------------------------------------------*/
		char  *action_str = (new_attr & (AT_EQUIV | AT_INIT)) ? new_str : old_str;
		char  *object_str = (new_attr & (AT_EQUIV | AT_INIT)) ? old_str : new_str;
		
		errmsg ("cannot %s %s: %q", action_str, object_str, name);
		/*}}}*/
	}
	else if (either_attr & AT_TARGET) {
		/*{{{  e.g. "named constant 'P' cannot be a pointer target"*/
		
		if (either_attr & AT_POINTER)
			/*------------------------------------------------------*/
			/* Have a special errmsg for this case, since		*/
			/*	"pointer 'X' cannot be a pointer target"	*/
			/* is misleading!					*/
			/*------------------------------------------------------*/
			errmsg ("cannot specify the TARGET attribute for a pointer: %q", name);
		
		else {
			/*------------------------------------------------------*/
			/* 'object_str' is an entity type.  Skip over its	*/
			/* leading article ("a ", "an " or "the ").		*/
			/*	N.B. This assumes that *all* strings associated	*/
			/* with flags AT_OBJECT begin with "a", "an" or "the"!	*/
			/*------------------------------------------------------*/
			char  *object_str = (new_attr & AT_TARGET) ? old_str : new_str;
		
			while (*object_str++ != ' ')
				;
		
			errmsg ("%s %q cannot be a pointer target", object_str, name);
		}
		/*}}}*/
	}
	else if (old_attr & AT_INTENT) {
		/*{{{  e.g. "cannot specify INTENT for a dummy pointer: 'X'"*/
		/*--------------------------------------------------------------*/
		/* N.B. 'new_attr' can't be AT_INTENT!  Therefore, 'old_attr'	*/
		/* is AT_INTENT.						*/
		/*	We know that the object is a dummy argument, as		*/
		/* otherwise AT_INTENT wouldn't be set.  However,		*/
		/* 'incompatible_attrs' didn't have AT_DUMMY set, otherwise	*/
		/* 'old_attr' would have been set to AT_DUMMY in preference	*/
		/* to AT_INTENT.						*/
		/*	The only 'new_attr's that conflict with AT_INTENT but	*/
		/* not AT_DUMMY are:						*/
		/*								*/
		/*	new_attr = AT_POINTER, AT_FUNC, AT_SUBR, AT_EXTERNAL	*/
		/*								*/
		/* so 'new_attr' is one of these!				*/
		/*--------------------------------------------------------------*/
		errmsg ("cannot specify INTENT for a dummy %s: %q",
		         (new_attr == AT_POINTER) ? new_str : "procedure", name);
		/*}}}*/
	}
	else if (either_attr & AT_SAVE) {
		/*{{{  e.g. "cannot SAVE an automatic array"*/
		/*--------------------------------------------------------------*/
		/* 'object_str' is 'a dummy argument' or an entity type.	*/
		/*--------------------------------------------------------------*/
		char  *object_str = (new_attr & AT_SAVE) ? old_str : new_str;
		
		errmsg ("cannot SAVE %s: %q", object_str, name);
		
		/*}}}*/
	}
	else if (either_attr & AT_ACCESS) {
		/*{{{  e.g. "cannot specify PUBLIC or PRIVATE access for a module: 'M'"*/
		/*--------------------------------------*/
		/* 'object_str' is an entity type.	*/
		/*--------------------------------------*/
		char  *object_str = (new_attr & AT_ACCESS) ? old_str : new_str;
		
		errmsg ("cannot specify PUBLIC or PRIVATE access for %s: %q",
		         object_str, name);
		/*}}}*/
	}
	else if (old_attr & (AT_DUMMY | AT_RESULT_VAR)) {
		/*{{{  e.g. "dummy argument 'D' cannot be an allocatable array"*/
		/*--------------------------------------------------------------*/
		/* N.B. 'new_attr' can't be AT_DUMMY or AT_RESULT_VAR, so it	*/
		/* must be 'old_attr'!  Skip over 'old_str's leading article,	*/
		/* so it becomes "dummy argument" or "function result variable".*/
		/*	 'new_str' is an object type.				*/
		/*--------------------------------------------------------------*/
		while (*old_str++ != ' ')
			;
		
		errmsg ("%s %q cannot be %s", old_str, name, new_str);
		/*}}}*/
	}

/*{{{  #undefs*/
#undef  AT_OBJECT
#undef  AT_IN_PLACE
#undef  AT_OTHER_ATTRS
/*}}}*/
}
/*}}}*/
/*{{{  shape_errmsg (sp, str_array, compatible_shape) -- ok*/
shape_errmsg (sp, str_array, compatible_shape)
/*------------------------------------------*/
	Symbol  *sp;
	char    *str_array;
	int      compatible_shape;
/*--------------------------------------------------------------*/
/*	Called by 'build_attrs ()' and 'install_attr' to report	*/
/* an error concerning an inappropriate array shape.		*/
/*--------------------------------------------------------------*/
{
	begin_errmsg (str_array);
	if (sp)
		cont_errmsg (" %q must have ", sp->name);
	else
		cont_errmsg (" must have ");

	switch  (compatible_shape & ~INVALID_SHAPE) {
	case CONST_SHAPE :
		end_errmsg (str_const_shape);
		break;

	case (CONST_SHAPE | DEFERRED_SHAPE) :
		end_errmsg (str_const_def_shape);
		break;

	case DEFERRED_SHAPE :
		end_errmsg (str_deferred_shape);
		break;

	case EXPL_SHAPE :
		end_errmsg (str_expl_shape);
		break;
	}
}
/*}}}*/

/*{{{  bool  mustbe_scalar_var (sp, datatype, var_str) -- ok*/
bool  mustbe_scalar_var (sp, datatype, var_str)
/*-------------------------------------------*/
	Symbol  *sp;
	int32    datatype;	/* a 'TY_BASIC_TYPE' flag or flags */
	char    *var_str;	/* string describing the context of
				     the var -- for errmsgs */
/*--------------------------------------------------------------*/
/*	Check that 'sp' is a scalar variable with datatype	*/
/* 'datatype'.  If so return TRUE, otherwise give an errmsg and	*/
/* return FALSE.						*/
/*--------------------------------------------------------------*/
{
	int32  sp_type;

	if (sp->attrs & AT_NONVAR) {
		errmsg ("%s %q must be a variable", var_str, sp->name);
		return FALSE;
	}

	sp_type = lookup_or_install_type (sp);	/* ...N.B. only do this if
						   'sp' *is* a variable! */
	if ( !(sp->attrs & AT_VAR))
		sp->attrs |= AT_NORMAL_VAR;	/* no 'attrs' flags can
						   conflict with this! */
	if (sp->arp) {
		errmsg ("%s %q must be scalar", var_str, sp->name);
		return  FALSE;
	}
	if ( ! (sp_type & (datatype | TY_NONE)) ) {
		begin_errmsg ("%s %q must ", var_str, sp->name);
		switch (datatype) {
		case  TY_INTEGER :
			end_errmsg ("have integer type");
			return  FALSE;

		case  TY_INT_REAL_DP :		/* DO / implied DO var */
			if (sp_type & TY_COMPLEX)
				end_errmsg ("not have complex type");
			else
				end_errmsg ("have numeric type (preferably integer!)");
			return  FALSE;
		}
	}
	return  TRUE;
}
/*}}}*/


/*{{{  *** OLD STUFF (for reference) -- check_id_class_in_expr(), argmatch() ****/
#if 0
/*{{{  check_idclass_in_expr (Symbol *sp, int *ep)*/
bool  check_idclass_in_expr (sp, ep)	/*--- OK ---*/
/*--------------------------------*/
	Symbol  *sp;
	int  *ep;	/* pointer to 'exprtype' */
/*--------------------------------------------------------------*/
/*  Checks that an ID has an appropriate 'class' for use in	*/
/*  a given expression (and if so, performs any other actions	*/
/*  that may be necessary, eg, calling 'install_impDO_var' or	*/
/*  setting the AT_AUTO bit of 'exprtype'.			*/
/*--------------------------------------------------------------*/
{
	/*----------------------------------------------*/
	/*  here, 'exprtype' masks off any 'attr' bits	*/
	/*  that may be optionally set:			*/
	/*----------------------------------------------*/
	int   exprtype = (*ep) & ~AT_ATTRS;
	int   sp_class = (sp->attrs & AT_ENTITY);

	/*** *ep &= ~AT_EVAL;	***/ /* switch off expression evaluation */

	switch (exprtype) {
	case  IMPDO_EXPR:
	/*-------------*/
		switch (sp_class) {
		case  AT_NORMAL_VAR:
			install_impDO_var (sp, DO_REF);
		case  AT_PARAM:
			return  TRUE;
		default:
			errmsg ("%q is not an implied-DO variable or named constant", sp->name);
			return  FALSE;
		}
	case  INIT_EXPR:
	/*------------*/
		switch (sp_class) {
		case  AT_PARAM:
			return  TRUE;
		default:
			errmsg ("%q is not a named constant", sp->name);
			return  FALSE;
		}
	case  SPEC_EXPR:
	/*------------*/
		switch (sp_class) {
		case  AT_DUMMY:
		case  AT_IN:
			/*--------------------------------------*/
			/* local array will be automatic	*/
			/*--------------------------------------*/
			/****  *ep |= AT_AUTO; ****/
		case  AT_PARAM:
			return  TRUE;
		case  AT_OUT:
			errmsg ("dummy argument %q is undefined on entry to the subprogram, as it has INTENT (OUT)", sp->name);
			return  FALSE;
		default:
			errmsg ("%q is not a named constant or dummy argument", sp->name);
			return  FALSE;
		}
	default:
	/*----*/
		errmsg ("COMPILER ERROR (function \'check_id_in_expr\'): unexpected expression flags = %d -- PLEASE REPORT!", exprtype);
		return  FALSE;
	}
}
/*-----------------------------------------------------------------------*/
/*}}}*/
#endif

/*{{{  bool argmatch (Symbol *proc, *actual, *dummy) *** UNFINISHED! ****/
bool  argmatch (proc, actual, dummy)
/*--------------------------------*/
	Symbol  *proc, *actual, *dummy;
{
	return TRUE;
}
/*
	int32  dum_attrs = dummy->attrs;

	if ( !actual)
		if (dum_attrs & AT_OPTIONAL) {
			printf (" I$UNDEF, .FALSE.,");
			return  TRUE;
		}
		else {
			errmsg ("subroutine %q has no actual argument corresponding to non-optional dummy argument %q", proc->name, dummy->name);
			printf (" I$UNDEF,");
			return  FALSE;
	}
	else {
		int  act_attrs = actual->attrs;

dummy could be:
--subr
--function, if so, check type & array & distr (not arguments!!)
--external (then any subr/fn will do)
*/
/*}}}*/
/*}}}*/

/* TO DO NEXT -- ENSURE THAT 'install_attr()' CALLED WHENEVER NECESSARY */
/*}}}*/
